 /**Class holds data for FracLac.
        * Holds fractal dimension and 
         *lacunarity (&lambda;) values at each grid position and
        * &epsilon;.  Lacunarity is calculated using 
        * the probability distribution's
        * {@link FLstatistics#CalculateCVsqForMassesAtEpsilon 
        *squared coefficient of variation} (cv<sup>2</sup>=
        * &sigma;/&mu;).  For binary scans, 
        *the probability distribution is found from a frequency 
        *histogram made using the area at &epsilon; 
        * (side<sup>2</sup>) as the maximum bin with 
        * increments of 1 between bins.  For grayscale scans, 
        * the maximum bin is the maximum intensity 
         * datum times (side<sup>2</sup>),
        * with increments of 1 between bins.
        *@see FLAnalyzer.FLDataProcessor*/
package FLAnalyzer;
import FLUtilities.FLstatistics;   
class FLData
{        
/**Constructor takes no parameters.*/          
FLData(){};              

//arrays and variables 
public  String [][] BinnedFrequencies,  BinnedMasses,
        UnbinnedFrequencies, UnbinnedMasses;  
         
public FLstatistics CountCVStats, EmptiesCVStats,
        EMassLacAtGridstats, CountDFStats , 
        countYinterceptStats , CountPrefactorStats ;

public   float [] LDF, countDF, cprefactor, countDFyint,
        countDFrsq,  countDFstderr, Smootheddf, Smoothedrsq, 
        SmoothedStdderr, Smoothedyint;

public float [][] EmptiesAtEpsilon, CountAtEpsilon,
        SmoothedCountSizes, SmoothedCount;
/**Holds a measure of the pixel distribution. Is for calculating 
* lacunarity (&Lambda; or &lambda;)  based on the {@link 
* FLUtilities.FLstatistics#CoefVar coefficient of variation} in the
* arrangement of pixels from box counting.  Lacunarity measures 
* heterogeneity or inhomogeneity essentially, and varies with the
* size of the sampling unit. Data for this value are based on the 
* number of pixels in each box that was placed on an image at an 
* &epsilon; during standard non-overlapping {@link FLMain
* FLMain#StartBoxCounts box counting} or {@link 
 *FLMain#SLACScan sliding box counting}. For each covering with 
 * an &epsilon;, the CV<sup>2</sup> is {@link 
 * FLUtilities.FLstatistics#CoefVar calculated} as:<ul><LI> 
 * (&sigma;/&mu;)<sup>2</sup></LI></ul><p>This is reported 
 * averaged over  all &epsilon;s for each location, averaged again 
 * over all locations  (with multiple locations), and as the slope of the
 * log log plot of ln &lambda;/ln &epsilon;.</p> Arrays with E or 
 * Empties in their name hold data for the entire image space, 
 * whereas those with F hold data for the part of the  image tested
 * that contained at least one foreground pixel.  Arrays with BDP in 
 * their names hold data from a {@link 
 * FLUtilities.FLutil#BinnedProbabilityStats binned probability 
 * distribution}; otherwise the probabilities are {@link 
 * FLUtilities.FLstatistics#cvsq not binned}. 
* @see "T.G. Smith, Jr., G.D.data. Lange and W.B. Marks, Fractal 
 * Methods and Results in Cellular Morphology, J. Neurosci. 
 * Methods,  69:1123-126, 1996." */    
public float [][] MassLacStdDevAtEpsilon, MassLacAtEpsilon,    
        MassLacMeanAtEpsilon ,     BPDLacAtEpsilon,   
        cvsqBPDLacAtEpsilon,       StdDevOfBPDAtEpsilon   ,
        MeanOfBPDAtEpsilon   ,    EBPDLacAtEpsilon,   
        EcvsqBPDLacAtEpsilon,       EStdDevOfBPDAtEpsilon   ,
        EMeanOfBPDAtEpsilon   ,    EMassStdDevAtEpsilon,
        EMassLacAtEpsilon,     EMassMeansAtEpsilon ;      
/**Holds a measure of the pixel distribution averaged for each 
 * {@link FLVars#GridPositions grid position} from the value stored
 * for lacunarity (&Lambda; or &lambda;). Is based on the {@link 
 * FLUtilities.FLstatistics#CoefVar coefficient of variation} in the
 * arrangement of pixels from box counting.  Lacunarity measures 
 * heterogeneity or inhomogeneity essentially, and varies with the
 * size of the sampling unit. Data for this value are based on the 
 * number of pixels in each box that was placed on an image at an 
 * &epsilon; during standard non-overlapping {@link 
 * FLMain#StartBoxCounts box counting} or  {@link 
 * FLMain#SLACScan sliding box counting}. For each covering with
 * an &epsilon;, the CV<sup>2</sup> is {@link 
 * FLUtilities.FLstatistics#CoefVar calculated} as:<ul><LI> (
 * &sigma;/&mu;)<sup>2</sup></LI></ul><p> This is {@link 
 * GUI.FL_#WriteScanData reported} averaged over all &epsilon;s 
 * for each location, averaged again over all locations  (with multiple
 * locations), and as the slope of the log log plot of ln &lambda;/ln
 * &epsilon;.</p>Arrays with E or Empties in their name hold data
 * for the entire image space, whereas those with F hold data for the
 * part of the  image tested that contained at least one foreground 
 * pixel.  Arrays with BDP in their names hold data from a {@link 
 * FLUtilities.FLutil#BinnedProbabilityStats binned probability 
 * distribution}; otherwise the probabilities are {@link 
 * FLUtilities.FLstatistics#cvsq not binned}. 
 * @see "T.G. Smith, Jr., G.D.data. Lange and W.B. Marks, Fractal 
 * Methods and Results in Cellular Morphology, J. Neurosci. 
 * Methods, 69:1123-126, 1996." */   
float [] BPDLacAtGrid,   MassLacAtGrid, VariationInMassLacAtGrid,
        VariationInBPDLacAtGrid,  EBPDLacAtGrid, 
        EVariationInBPDLacAtGrid, EmptiesCVSqAtGrid, 
        CountCVsqAtGrid,   EMassLacAtGrid,
        VariationInEMassLacAtGrid; 
public float [] massDF, mprefactor, massDFyint, massDFrsq, 
        massDFstderr;
public FLstatistics MassLacAtGridstats,MassDFStats ,
        MassPrefactorStats;
/**Holds values for lacunarity for grayscale images, from the 
 * squared coefficient of variation in the intensity of pixels ((max-min
 * )*size<sup>2</sup> or (max-min+1) )from box counting.  Is 
 * based on the {@link FLUtilities.FLstatistics#CoefVar coefficient 
 * of variation} in the arrangement of pixels from box counting.  
 * Lacunarity measures heterogeneity or inhomogeneity essentially,
 * and varies with the size of the sampling unit. Data for this value 
 * are based on the number of pixels in each box that was placed on
 * an image at an &epsilon; during standard non-overlapping {@link
 * FLMain#StartBoxCounts box counting} or {@link 
 * FLMain#SLACScan sliding box counting}. For each covering with
 * an &epsilon;, the CV<sup>2</sup> is {@link 
 * FLUtilities.FLstatistics#CoefVar calculated} as:<ul><LI>
 * (&sigma;/&mu;)<sup>2</sup></LI></ul><p> This is {@link 
 * GUI.FL_#WriteScanData reported} for each &epsilon;, averaged 
 * over  all &epsilon;s for each location, averaged again over all 
 * locations (with multiple locations), and as the slope of the log log 
 * plot of ln &lambda;/ln &epsilon;.</p>Arrays with E or Empties in 
 * their name hold data for the entire image space, whereas those 
 * with F hold data for the part of the  image tested that contained at
 * least one foreground pixel.  Arrays with BDP in their names hold 
 * data from a {@link FLUtilities.FLutil#BinnedProbabilityStats 
 * binned probability distribution}; otherwise the probabilities are 
 * {@link FLUtilities.FLstatistics#cvsq not binned}. 
 * @see "T.G. Smith, Jr., G.D.data. Lange and W.B. Marks, Fractal 
 * Methods and Results in Cellular Morphology, J. Neurosci. Methods
 * , 69:1123-126, 1996." */   
public  float [][] graySumOfBoxMassesAtEpsilon, 
        grayCountAtEpsilon,   grayMassLacMeanAtEpsilon,       
        grayMassLacAtEpsilon,    grayMassLacStdDevAtEpsilon;
public float []grayMassLacAtGrid,  grayVariationInMassLacAtGrid,
        graymassDF,   graymassDFyint, graymassDFyintprefactor,
        graymassDFstderr, graymassDFrsq;    
public  FLstatistics grayMassYintStats , 
        grayMassDFrsqStats,  grayMassStdErrStats, 
        grayMassDFStats ,  grayMassLacAtGridstats;     

    //each stat needs a holder for itself, the mean, and the stdev
    //epsilon, one for the mean at each grid, and a stats instance 
    //from that        
public   FLstatistics BPDLacAtGridStats, EBPDLacAtGridStats;  
public  float [] mincover , avgcover ;
public FracStats mins, avgs, SmoothedCountStats;
public  float [][] minmasses;
public  int [] minBoxes;
    /**@see FLAnalyzer.FLDataProcessor#getBinarystats*/
public  float CountPrefactorLacOverAllGrids=0f, 
            MassPrefactorLacOverAllGrids, 
            grayPrefactorLacOverAllGrids =0f;

}
                 