package FLAnalyzer;
import FLUtilities.FLstatistics;
import FLUtilities.FLutil;
import ij.IJ;
import ij.text.TextWindow;
/**Class calculates statistics and stores data for writing strings
* for FracLac.*/
class FLDataProcessor
{  
/*Constructor takes no parameters, does nothing*/
public FLDataProcessor(){}
     int grids;     
     FLMain F = null;
     FLVars V = null;
     static String tab = "\t", newline = "\n";
/**Calculates statistics using previously filled
* {@link FLMain#Pix counting array}.*/          
public void ProcessData (
    FLMain FLMAIN,
    FLVars VARS)
{  
           V=VARS;   
           F=FLMAIN;
           grids =V.GridPositions;
           
           if(V.CheckPixRatio) RemoveEdges ();
           data = new FLData();          
           BinnedProbabilityLacunarity ();
           if(!V.isGray())EBinnedProbabilityLacunarity ();
           if(V.isGray()) FindGrayDf ();
           else 
            {
                getBinarystats();
                getEstats();           
            }
        IJ.showStatus ("Done getting stats.");
       if(V.WriteRawData)recordRawData ();
       if(V.printFrequencies)StoreUnbinnedFrequenciesAndMasses();
}

/**Returns false or else true if the ratio of foreground pixels (z)
 * in the box of size boxsize is less than {@link FLVars#FACTOR 
 * the set value}.
 * @param z int for number of foreground pixels
 * @param boxsize int for box's side length*/
boolean CheckPixelRatio(float z, float boxsize)
{
    boolean toosparse=false;
    if ( (float)(z/Math.pow (boxsize, 2)) <V.getFACTOR ())
    toosparse=true;                  
    return toosparse;
}
/**Changes the Pix and boxsizes array to exclude any boxes that are
 * discounted using the pixel ratio.*/
void RemoveEdges()
{
    //to determine if a count should be removed as an "edge",
    //find the ratio of pixels to box size - correct for edge effects  
    int B[][]= new int [grids][];
    float [][][] P = new float [grids][][]; 
    //make an empty copy of box sizes
    for (int g = 0; g < grids;g++)
      B[g]=FLutil.newArray (F.BOXSIZES[g].length, 0);
    //make an empty copy of pixels
    for (int g = 0; g < grids; g++)
    {
        P[g]=new float [F.Pix[g].length][];
        for (int b= 0; b<F.Pix[g].length; b++)
       {
                P[g][b]=FLutil.newArray(F.Pix[g][b].length, 0f);        
        }
    }
    int countAtEpsilon [][] = new int [grids][];
    for (int g = 0 ; g < grids; g++)
    {
        countAtEpsilon[g]=new int [F.Pix[g].length];
        for (int b = 0; b < F.Pix[g].length; b++)
        {  
            countAtEpsilon[g][b]=0;           
            for (int i =0; i < F.Pix[g][b].length; i++)
            {
                if(!CheckPixelRatio(F.Pix[g][b][i], F.BOXSIZES[g][b]))
                {
                    P[g][b][i]=F.Pix[g][b][i];
                    countAtEpsilon[g][b]++; 
                }
               
            } if (countAtEpsilon[g][b]>=1) B[g][b]=F.BOXSIZES[g][b];            
        }
    }
    
    //now there is an array of Box sizes and an arry of masses 
    //with zero values at the discardable positions
    //make a new copy of each without zeros;
    for (int g = 0 ; g < grids; g++)
    {      
        int boxes = FLutil.countGreaterOrEqual(B[g], B[g].length, 1);                 
        F.BOXSIZES[g]= new int [boxes];    
        int b = 0;
        for (int z = 0; z < B[g].length; z++)
        {
            if (B[g][z]>=1) 
            {
               F.BOXSIZES[g][b]=B[g][z];             
               F.Pix[g][b]=new float [countAtEpsilon[g][z]];
               int i = 0;
               for ( int q = 0; q <P[g][b].length; q++)
               {
                   //fill the new pix array
                   if (P[g][b][q]>=1)
                   {
                       F.Pix[g][b][i]=P[g][b][q];
                       i++;
                   }
                   
               }
                b++;
            }
        }
    }    
       
    float d =  (float)V.GreaterOfHtAndWd;   
    F.Epsilons = new float [F.BOXSIZES.length][];
    for (int i = 0; i < F.BOXSIZES.length; i++)
    { 
        F.Epsilons[i] = new float [F.BOXSIZES[i].length];
        for (int j =0; j < F.BOXSIZES[i].length; j++)
        F.Epsilons[i][j]=(float )F.BOXSIZES[i][j]/d;
    }
    //only use a box if it had more than the pixel ratio
    //if it is less than the pixel ratio, remove that mass
    //if, at the end there is a box size that has no count, 
    //remove it from the series and remake the series
    //report this data separately because the box sizes 
    //may change with each grid location    
    
}

/**Stores data for processing scans.
@see FLAnalyzer.FLData
@see FLAnalyzer.FLDataProcessor
@see GUI.FL_#WriteScanData
@see FLMain#WriteDataStringsForAllLocations*/   
FLData data=null;      
 void InitializeGrayArrays(int grids)
 {
        data.graySumOfBoxMassesAtEpsilon=new float[grids][ ];
        data.grayCountAtEpsilon= new float [grids][ ];

        data.grayMassLacMeanAtEpsilon=new float[grids][ ];
        data.grayMassLacAtEpsilon=new float[grids][ ];
        data.grayMassLacStdDevAtEpsilon=new float[grids][ ];
        data.grayMassLacAtGrid=new float [grids];                 
        data.grayVariationInMassLacAtGrid=new float [grids];

        data.graymassDF=new float [grids];
        data.graymassDFyint=new float [grids];                           
        data.graymassDFyintprefactor=new float [grids];
        data.graymassDFstderr=new float [grids];                              
        data.graymassDFrsq=new float [grids];                      
}
 void InitializeGrayArraysAtEpsilon(int g, int e)
{                       
data.graySumOfBoxMassesAtEpsilon[g]=new float[e];
data.grayCountAtEpsilon[g]=new float[e];
data.grayMassLacMeanAtEpsilon[g]=new float[e];
data.grayMassLacAtEpsilon[g]=new float[e];
data.grayMassLacStdDevAtEpsilon[g]=new float[e];
}
/**Stores the grayscale Df in an array {@link FLMain#Pix 
 * the scan array}
 * now holds at each [grid][boxsize][i] either ((max-min)*s*s) or 
 * (max-min+1) from each (i,j) at that box size, where s = box size.
 * To find the df,  sum each array so that there is a  single input per
 * &epsilon; then take the slope of the log &#8721;I vs log &epsilon;.
 *This function calculates the fractal dimension  for gray scale
 * images using  the differential box counting method and another 
 * method, both using the maximum-minimum gray scale values. 
 * The first assumes volume, v, can be defined  over a gray scale
 * image, corresponding to the intensity of the  F.Pixels, for each box 
 * of size r in a grid placed over the image.  The volume can be 
 * deduced as r*r*p.  Then  S= Slope (ln v/ln &;epsilon;) using {@link 
 * FLAnalyzer.FracStats#SlopeOfPowerRegression  the slope} from 
 * a power regression based  on &#8721;p*r<sup>2</sup>, where 
 * r is relative size and p is max-min.. Then, the D<sub>
 * F-GRAYSCALE</sub>=3-(S/2). The other method uses the slope
 * of  the ln(max-min +1)/ln (1/r), where r is scale/Maximum Image 
 * Dimension and D<sub>f</sub> = slope.*/
public void FindGrayDf()
{
        InitializeGrayArrays(grids);         
        IJ.showStatus("Finding grayscale fractal dimension.");
        //------      for every grid position, then for each epsilon   ---/
        //-------   at that position, find the sum and lacunarity     ---/
        //--------        stats from all boxes at that epsilon   -------------/
        for (int g=0; g<grids; g++)
        { 
            InitializeGrayArraysAtEpsilon(g, F.Epsilons[g].length); 
            for (int b = 0; b<F.Epsilons[g].length; b++)
                {            
                    int l = F.Pix[g][b].length;
                    FLstatistics stats= new FLstatistics(F.Pix[g][b], l);
                    data.graySumOfBoxMassesAtEpsilon[g][b]=stats.sum;  
                    data.grayMassLacMeanAtEpsilon[g][b]=stats.mean;
                    data.grayMassLacAtEpsilon[g][b]=stats.cvsq;
                    data.grayMassLacStdDevAtEpsilon[g][b] =stats.stddev;
                    data.grayCountAtEpsilon[g][b]=l;
                } 
            IJ.showStatus ("Finding grayscale Df  "+g+"/"+grids);
        }
        //-----  Calculate the fractal dimension for grayscale data  -------/
        for (int g=0; g < grids; g++)
        {
            float [] x=(V.grayscantype==V.DIFFERENTIALBC)?
            FLutil.InverseFloatArray (F.Epsilons[g]):
            FLutil.IntToFloatArray (F.BOXSIZES[g]);
            //Use epsilon inverted for the differential method,
            //or else a float cast of box size for the 2d variation
            FLAnalyzer.FracStats FS =new FLAnalyzer.FracStats();
            float [] D = FS.PowerRegression
                    (data.graySumOfBoxMassesAtEpsilon[g], x, x.length);
            //for the log-log plot, use 
            //the sum of masses which are (Max-min+1) or
            //else (max-min)*s*s where s is box size.
            float d =D[FS.SLOPEindex];
            if (Float.isNaN (d)||Float.isInfinite (d))d=0f;                     
            data.graymassDF[g]=
                    (V.grayscantype==V.DIFFERENTIALBC)
            ?   d  :3f-(d/2f); 
            //the actual fractal dimension in this case is 
            //the slope of the log log plot for the differential box
            //counting method, or 3-(2/slope) for the 2d variation
            data.graymassDFyint[g]=D[FS.YINTindex];                           
            data.graymassDFyintprefactor[g]
                    =D[FS.PREFACTORindex];                                                
            data.graymassDFstderr[g] =D[FS.STDERRindex];                              
            data.graymassDFrsq[g] =D[FS.RSQindex];   
            IJ.showStatus ("Finding grayscale lacunarity "+g+"/"+grids);             
            FLstatistics S = new FLstatistics(
                    data.grayMassLacAtEpsilon[g],  
                    data.grayMassLacAtEpsilon[g].length);
            data.grayMassLacAtGrid[g]=S.mean;
            data.grayVariationInMassLacAtGrid[g]=S.cvsq;     
        }
        float Prefactorsmean =
                FLutil.MeanOfArray (data.graymassDFyintprefactor);
        float avPrefactorLac=0;
        for (int m = 0; m < grids; m++)
        {
                avPrefactorLac  = avPrefactorLac +
                        ((float)Math.pow
                        (((data.graymassDFyintprefactor[m]/
                        Prefactorsmean)- 1.0f), 2));
        }
        data.grayPrefactorLacOverAllGrids=
                avPrefactorLac/(float)this.grids;          
        data.grayMassLacAtGridstats=new FLstatistics
                (data.grayMassLacAtGrid, grids);                       
        data.grayMassDFStats = 
                new FLstatistics(data.graymassDF, grids);
         data.grayMassDFrsqStats = 
                new FLstatistics(data.graymassDFrsq, grids);
         data.grayMassStdErrStats = 
                new FLstatistics(data.graymassDFstderr, grids);
         data.grayMassYintStats = 
                new FLstatistics(data.graymassDFyint, grids);
         data.grayMassLacAtGridstats=new FLstatistics(
                 data.grayMassLacAtGrid, grids);
}

void  initializeBinaryArrays()
 {
    data.MassLacStdDevAtEpsilon= new float [grids][ ]; 
    data.MassLacAtEpsilon=new float [grids][ ];        
    data.MassLacMeanAtEpsilon =    new float [grids][ ];
    data.MassLacAtGrid= new float [grids];
    data.VariationInMassLacAtGrid=new float [grids];
    data.SmoothedCountSizes =new float [grids][ ];
    data.SmoothedCount=new float [grids][ ];
    data. massDF=new float [grids];
    data. mprefactor=new float [grids];
    data.massDFyint=new float [grids];
    data.massDFrsq=new float [grids];
    data.massDFstderr=new float [grids];                   

    data.CountAtEpsilon=new float [grids][ ];
    data.countDF=new float [grids];
    data.LDF=new float [grids];
    data.cprefactor=new float [grids];
    data.countDFyint=new float [grids];
    data.countDFrsq=new float [grids];
    data.countDFstderr=new float [grids];
    data.CountCVsqAtGrid = new float [grids];

    data.Smootheddf=new float [grids];
    data.Smoothedrsq=new float [grids]; 
    data.Smoothedyint=new float [grids]; 
    data.SmoothedStdderr=new float [grids];
    data.SmoothedCountStats=new FracStats();            
 }
void InitializeBinaryArraysAtEpsilon(int g, int e)
{                
                data.MassLacStdDevAtEpsilon[g]= new float [e];
                data.MassLacAtEpsilon[g]=new float [e];       
                data.MassLacMeanAtEpsilon[g]= new float [e];                        
                data.SmoothedCountSizes[g]=new float [e];
                data.SmoothedCount[g]=new float [e];
                data.CountAtEpsilon[g]=new float [e];
  
}
/**This is the data crunching function that gets fractal dimension 
* and lacunarity statistics from the {@link FLMain#Pix raw data 
 * array}and stores the data in various arrays. Uses {@link 
 * FLstatistics stats} class to get the {@link FLstatistics#cvsq 
 * coefficient of variation (&sigma;/&mu;)},{@link 
 * FLstatistics#stddev standard deviation (&sigma;)},and {@link 
 * FLstatistics#mean mean (&mu;)}, and {@link 
 * FLAnalyzer.FracStats FracStats} to find the fractal dimension
 * (D<sub>f</sub>), r<sup>2</sup>, y-intercept and {@link 
 * FLAnalyzer.FracStats#FractalStatsF prefactor}, and standard 
 * error.
* @see FLAnalyzer.FLData#MassLacStdDevAtEpsilon
* @see FLAnalyzer.FLData#MassLacAtEpsilon        
* @see FLAnalyzer.FLData#MassLacMeanAtEpsilon     
* @see FLAnalyzer.FLData#MassLacAtGrid 
* @see FLAnalyzer.FLData#VariationInMassLacAtGrid
* @see FLAnalyzer.FLData#massDF
* @see FLAnalyzer.FLData#massDFyint
* @see FLAnalyzer.FLData#massDFrsq
* @see FLAnalyzer.FLData#massDFstderr                   
* @see FLAnalyzer.FLData#CountAtEpsilon
* @see FLAnalyzer.FLData#countDF
* @see FLAnalyzer.FLData#LDF
* @see FLAnalyzer.FLData#countDFyint
* @see FLAnalyzer.FLData#countDFrsq
* @see FLAnalyzer.FLData#countDFstderr
* @see FLAnalyzer.FLData#CountCVsqAtGrid*/
public void getBinarystats()
{
    initializeBinaryArrays();    
    //===================================//
    //               STORE STATS FROM PIXEL DATA                             //
    //              AT EACH GRID LOCATION AND BOX SIZE               //
    //===================================//
    for (int g = 0; g <grids; g++)
    {
        InitializeBinaryArraysAtEpsilon(g, F.Epsilons[g].length);
        for (int b = 0; b < F.Epsilons[g].length; b++)        
       {
            IJ.showStatus("Getting stats for "+g+": "+b);
            FLstatistics FS
                 = new FLstatistics(F.Pix[g][b], F.Pix[g][b].length);
            data.MassLacMeanAtEpsilon[g][b] =FS.mean;
            data.MassLacAtEpsilon[g][b]=FS.cvsq;
            data.MassLacStdDevAtEpsilon[g][b]=FS.stddev;
            data.CountAtEpsilon[g][b]=F.Pix[g][b].length;
        }
    }    
    IJ.showStatus("Done getting data.");
    //===================================/
    //     CALCULATE THE DF AND LACUNARITY FROM DATA    /
    //                    one for each GRID location                                      /
    //===================================/
    for (int g=0; g<grids; g++)        
    {
        FracStats M=new FracStats(data.MassLacMeanAtEpsilon[g],
                F.Epsilons[g], F.Epsilons[g].length);                    
        data.massDF[g] = M.df; 
        data.mprefactor[g]=M.FractalStatsF;
        data.massDFyint[g]=M.FractalStatsYint;
        data.massDFrsq[g] = M.FractalStatsRsq;
        data.massDFstderr[g]=M.stderr;
        M=new FracStats(data.CountAtEpsilon[g], F.Epsilons[g], 
                F.Epsilons[g].length);                    
        data.countDF[g] =  M.df;
        data.cprefactor[g]=M.FractalStatsF;
        data.countDFyint[g]=M.FractalStatsYint;
        data.countDFrsq[g] = M.FractalStatsRsq;
        data.countDFstderr[g]=M.stderr;                      
        FLstatistics S=new FLstatistics(data.MassLacAtEpsilon[g],  
                data.MassLacAtEpsilon[g].length);
        data.MassLacAtGrid[g]= S.mean;
        data.VariationInMassLacAtGrid[g]=S.cvsq; 
        S= new FLstatistics(data.CountAtEpsilon[g], 
                data.CountAtEpsilon[g].length);
        data.CountCVsqAtGrid[g]=S.cvsq;                      
        float [] h= data.SmoothedCountStats.LinearSmoothStats
            (data.CountAtEpsilon[g], F.Epsilons[g], F.Epsilons[g].length);
        data.SmoothedCountSizes[g]=
                data.SmoothedCountStats.SmoothedSize;
        data.SmoothedCount[g]=
                data.SmoothedCountStats.SmoothedCount;
        data.Smootheddf[g]=
                h[data.SmoothedCountStats.DFSLOPEindex];
        data.Smoothedrsq[g]=h[data.SmoothedCountStats.RSQindex];
        data.SmoothedStdderr[g]=
                h[data.SmoothedCountStats.STDERRindex];
        data.Smoothedyint[g]=h[data.SmoothedCountStats.YINTindex];
    }    
    IJ.showStatus("Done df stats.");
    //====================================/
    //            SUMMARIZE RESULTS OVER MULTIPLE GRIDS         /
    //====================================/
    //GET MINIMUM COVER RESULTS
    FLUtilities.BoxCountHolder b 
            = FLutil.minCover (F.Pix, F.BOXSIZES, V.CheckPixRatio); 
    data.minBoxes = b.boxes; 
    data.mincover = b.counts;
    data.minmasses=b.masses;        
    data.mins=new FracStats (data.mincover, data.minBoxes, 
            data.minBoxes.length);    
    //====================================/
    //                     GET AVERAGE COVER RESULTS                            /
    //====================================/
    //          IF EACH GRID USED THE SAME ARRAY OF EPSILONS /
    //    FIND THE AVERAGE AT EACH EPSILON AND CALCULATE/
    //         THE AVERAGE STATS USING THAT NEW ARRAY             /
    //----------------------------------------------------------------------------------------/
   if(!V.CheckPixRatio)
   {
     data.avgcover= 
             FLutil.MeanMatchedArray (data.CountAtEpsilon, grids,
            data.CountAtEpsilon[0].length);
     data.avgs= new FracStats (data.avgcover, F.Epsilons[0], 
            F.Epsilons[0].length);
   }
    //-------------------------------------------------------------------------------------- /
    //   IF EACH GRID USED A DIFFERENT ARRAY OF EPSILONS /
    //              MAKE AVERAGES ARRAY                                                /
    //-------------------------------------------------------------------------------------- /
   else
   {
        FLUtilities.BoxCountHolder B 
                = FLutil.MeanOfUnMatchedArrays
                (data.CountAtEpsilon, F.BOXSIZES);
        data.avgcover=B.counts;
        data.avgs=new FracStats (data.avgcover, B.boxes,
                B.boxes.length);        
   } 
    IJ.showStatus("Done min cover.");        
    
    //GET PREFACTOR LACUNARITY OVER ALL GRIDS
    data.CountPrefactorStats=
            new FLstatistics(data.cprefactor,grids);        
    data.CountPrefactorLacOverAllGrids=0f;
    for (int m = 0; m < grids; m++)
    {
     data.CountPrefactorLacOverAllGrids =
             data.CountPrefactorLacOverAllGrids +((float)Math.pow
             ( ((data.cprefactor[m]/ data.CountPrefactorStats.mean)
             - 1.0f), 2));
    }
    data.CountPrefactorLacOverAllGrids = 
        data.CountPrefactorLacOverAllGrids/(float)grids;         
    data.MassPrefactorStats=new FLstatistics(data.mprefactor,grids);  
    data.MassPrefactorLacOverAllGrids=0f;
    for (int m = 0; m < grids; m++)
    {
        data.MassPrefactorLacOverAllGrids = 
             data.MassPrefactorLacOverAllGrids +((float)Math.pow
             (((data.mprefactor[m]/ data.MassPrefactorStats.mean)
             - 1.0f), 2));
    }
    data.MassPrefactorLacOverAllGrids = 
        data.MassPrefactorLacOverAllGrids/(float)grids;        
    //STORE STATS FOR COUNT AND MASS DFS AT EACH GRID
    data.CountDFStats = new FLstatistics(data.countDF, grids);
    data.MassDFStats = new FLstatistics(data.massDF, grids);
    data.countYinterceptStats =new FLstatistics(data.countDFyint,
        grids);        
    data.MassLacAtGridstats=new FLstatistics(data.MassLacAtGrid,
         grids);
    data.CountCVStats=new FLstatistics(data.CountCVsqAtGrid,
            grids);
    //STORE THE LAMBDA DF AT EACH GRID
    for (int i = 0; i < grids; i++)
    {
         FracStats s = new FracStats();
         float [] f = s.PowerRegression(data.MassLacStdDevAtEpsilon[i],
                 F.Epsilons[i],  F.Epsilons[i].length);
         float sig = f[s.DFSLOPEindex];
         f=s.PowerRegression(FLutil.Plus1(data.MassLacAtEpsilon[i]),
                 F.Epsilons[i], F.Epsilons[i].length);
         float lac = f[s.DFSLOPEindex];
         data.LDF[i]=sig-(lac/2.0f);             
    }        
}

void InitializeEArrays()
{
    data.EmptiesAtEpsilon=new float [grids][];
    data.EMassStdDevAtEpsilon=new float [grids][];
    data.EMassLacAtEpsilon=new float [grids][];        
    data.EMassMeansAtEpsilon = new float [grids][];   
    data.EmptiesCVSqAtGrid=new float [grids];
    data.EMassLacAtGrid=new float [grids];
    data.VariationInEMassLacAtGrid=new float [grids];
}
void InitializeEArraysAtEpsilon(int g, int e)
{    
    data.EmptiesAtEpsilon[g]=new float [e];
    data.EMassStdDevAtEpsilon[g]=new float [e];
    data.EMassLacAtEpsilon[g]=new float [e];    
    data.EMassMeansAtEpsilon[g]=new float [e];        
}
/**Calculates and stores statistics from raw data in {@link 
 * FLMain#Pix} and {@link FLMain#Empties} for values coded 
 * with "E" in the results file.  Calls {@link #AddZerosToArray 
 * padding} function to prepare the data, as in the gathering stage,
 * masses of zero were not included in the array. This is the data 
 * crunching function that gets fractal dimension and lacunarity 
 * statistics from the {@link FLMain#Pix raw data array}and stores
 * the data in various arrays.  Uses {@link FLstatistics stats} class 
 * to get the {@link FLUtilities.FLstatistics#cvsq coefficient of 
 * variation (&sigma;/&mu;)},{@link FLstatistics#stddev standard 
 * deviation (&sigma;)}, and {@link FLstatistics#mean mean (&mu;)
 * }, and {@link FLAnalyzer.FracStats FracStats} to find the fractal
 * dimension (D<sub>f</sub>), r<sup>2</sup>,  y-intercept and
 * {@link FLAnalyzer.FracStats#FractalStatsF prefactor}, and 
 * standard error.
 * @see #getBinarystats */ 
public void getEstats()
{            
    InitializeEArrays();    
    //fill arrays with statistics based on F.Pixel counts        
    for (int g = 0; g <grids; g++)
    {
        InitializeEArraysAtEpsilon(g, F.Epsilons[g].length);  
        for (int b = 0; b <F.Epsilons[g].length ; b++)
        {                                   
            //for lacunarity
            int l = F.Pix[g][b].length;
            IJ.showStatus("Getting E lacunarity for "+g+": "+b);
            FLstatistics FS=new FLstatistics( 
                    AddZerosToArray(F.Pix[g][b], l, (int)(l +F.Empties[g][b])),
                    (int)(l +F.Empties[g][b]));
            data.EMassLacAtEpsilon[g][b]=FS.cvsq;
            data.EMassStdDevAtEpsilon[g][b]=FS.stddev;
            data.EmptiesAtEpsilon[g][b]=  l +F.Empties[g][b];
            data.EMassMeansAtEpsilon [g][b]= FS.mean;                 
        }
    }
    IJ.showStatus("Done E lacunarity.");
    
    //make arrays of the fractal dimension and related stats
    //one for each location
    for (int g=0;g<grids;g++)        
    { 
        FLstatistics  S = new FLstatistics (data.EmptiesAtEpsilon[g], 
                data.EmptiesAtEpsilon[g].length);
        data.EmptiesCVSqAtGrid[g]=S.cvsq;
        S=new FLstatistics(data.EMassLacAtEpsilon[g], 
                data.EMassLacAtEpsilon[g].length);
        data.EMassLacAtGrid[g]=S.mean;
        data.VariationInEMassLacAtGrid[g]=S.cvsq;
    }    
    IJ.showStatus("Done E df stats.");    
    //Now make statistics summarizing results over multiple grids  
    data.EmptiesCVStats=new FLstatistics(data.EmptiesCVSqAtGrid,
            grids);
    data.EMassLacAtGridstats=new FLstatistics
            (data.EMassLacAtGrid, grids);                  
}     
             
     
/**Stores strings in {@link FLAnalyzer.FLData data instance} for
*UnbinnedFrequencies and UnbinnedMasses.  These are from the 
*{@link FLstatistics#CalculateFrequencyDistribution a 
*frequency distribution} of F.Pixels masses or intensities stored in 
*{@link FLMain#Pix the F.Pixel count array}. The strings are made 
*using a specialized {@link FLutil#MassAndFrequencyStrings
*utility function}.
*@see FLVars#printFrequencies*/ 
public void StoreUnbinnedFrequenciesAndMasses()
{
  if(!V.isMF()&&!V.SLAC)
  { 
      data.UnbinnedFrequencies =  new String[grids][];
      data.UnbinnedMasses = new String[grids][];             
      //get strings for frequencies and masses for regular 
      //box counts
      for (int g = 0; g < grids; g++)
      {
          for (int b = 0; b<F.Epsilons[g].length;  b++)
          {
              if (b==0)
              {
                  int e=F.Epsilons[g].length;
                  data.UnbinnedFrequencies[g] = new String[e];
                  data.UnbinnedMasses[g] = new String [e]; 
              }
               float Max = FLutil.maxArray (F.Pix[g][b]);
               float [] f = FLstatistics.CalculateFrequencyDistribution
                        ( F.Pix[g][b],  Max); 
               String [] MF=FLutil.MassAndFrequencyStrings (f);
               data.UnbinnedFrequencies[g][b]= MF[0];
               data.UnbinnedMasses[g][b]= MF[1];
             }
          }              
  }
}
/**Returns the passed integer array with 0s at the end, starting
 * from the passed value for length, up to the passed value for 
 * newlength.
 * @param array float array to add zeroes to at the end
 * @param length the position in the old array from which to start 
 * adding 0s
 * @param newlength the newlength desired
 * @return  a float[] that is the old array up to the length specified, 
 * then all 0s up to the newlength*/
public float [] AddZerosToArray (float[] array, int length, 
        int newlength)
{
    float [] newarray = new float [newlength];
    for (int i = 0; i < newlength; i++)
    {
         if (i<length) newarray[i]=array[i];
         else newarray[i]=0;
    }
    return newarray;
}  
void InitializeBPD (int grids)
{
        data.MeanOfBPDAtEpsilon   =new float [grids][];
        data.BPDLacAtEpsilon=     new float [grids][];
        data.cvsqBPDLacAtEpsilon=new float[grids][];
        data.StdDevOfBPDAtEpsilon=new float [grids][];        
        data.VariationInBPDLacAtGrid=new float [grids];
        data.BPDLacAtGrid=new float [grids];   
        data.BinnedFrequencies=new String [grids][];
        data.BinnedMasses=new String[grids][];
}
void InitializeBPDAtEpsilon(int g, int e)
{
    data.MeanOfBPDAtEpsilon[g]=new float [e];
    data.BPDLacAtEpsilon[g]=new float [e];
    data.cvsqBPDLacAtEpsilon[g]=new float [e];
    data.StdDevOfBPDAtEpsilon[g]=new float [e];      
    data.BinnedFrequencies[g]=new String [e];
    data.BinnedMasses[g]=new String [e];
}

/**Makes a {@link FLstatistics#binnedProbabilityDistribution 
 * binned probability distribution} from the stored {@link
 * FLMain#Pix scan array} and {@link 
 * FLutil#BinnedProbabilityStats calculates} statistics and prepares
 * data strings: {@link FLAnalyzer.FLData#MeanOfBPDAtEpsilon 
 * mean}, {@link FLAnalyzer.FLData#BPDLacAtEpsilon lacunarity}
 * , {@link FLAnalyzer.FLData#cvsqBPDLacAtEpsilon CV<sup>2
 * </sup>}, {@link FLAnalyzer.FLData#StdDevOfBPDAtEpsilon} 
 * standard  deviation (&sigma;), {@link 
 * FLAnalyzer.FLData#VariationInBPDLacAtGrid}, {@link 
 * FLAnalyzer.FLData#BPDLacAtGrid binned probability 
 * distribution &lambda;}, {@link 
 * FLAnalyzer.FLData#BinnedFrequencies}, {@link 
 * FLAnalyzer.FLData#BinnedMasses}.  The mean (&mu;) and CV
 * (&sigma;/&mu;) for the &lambda; statistic over all &epsilon; are
 * stored at each location, then &mu; and &sigma;/&mu;
 * <sup>2</sup> of the mean &Lambda; over all locations are 
 * calculated and reported.  If {@link FLVars#printFrequencies} is 
 * true, prepares a string of bins and a string of corresponding 
 * probabilities for printing.
 *@see #getBinarystats*/
private void BinnedProbabilityLacunarity()
{
    InitializeBPD(grids);    
    for (int g=0; g<grids; g++)
    {    
        InitializeBPDAtEpsilon(g, F.Epsilons[g].length);   
        for (int b = 0; b < F.Epsilons[g].length; b++)
        {                  
            //bins at 0; probabilities at 1 binnedmprob holds 
            //an array of the probabilities sorted into categories
            //instead of for
            //each individual value so it has two arrays, one for the value
            //and one for the frequency of that value
            float [] d=null;
            float[][] binnedmprob = FLstatistics.
                    binnedProbabilityDistribution(F.Pix[g][b], 
                    F.Pix[g][b].length,V.MaxFrequencies, false);
            float [] binnedPstuff = FLutil.BinnedProbabilityStats 
                    (binnedmprob, g);
            if (binnedPstuff==null)binnedPstuff=FLutil.newArray(5,0.0f);
            data.MeanOfBPDAtEpsilon[g][b]=binnedPstuff[0];
            data.StdDevOfBPDAtEpsilon[g][b] =binnedPstuff[1];
            data.cvsqBPDLacAtEpsilon[g] [b] =binnedPstuff[4];
            data.BPDLacAtEpsilon[g][b] =binnedPstuff[3];
            if(V.printFrequencies)
             {
                String [] S =FLutil.MassAndProbabilityStrings (
                    binnedmprob); 
                data.BinnedFrequencies[g][b]=S[1]==null?"":S[1];
                data.BinnedMasses[g][b]=S[0]==null?"":S[0];
               } 
            }
    }
    for (int g=0; g<grids; g++)
    {//make arrays that summarizes the pd statistics
        FLstatistics S = new FLstatistics(data.BPDLacAtEpsilon[g], 
                data.BPDLacAtEpsilon[g].length);
        data.BPDLacAtGrid[g]=S.mean;          
        data.VariationInBPDLacAtGrid[g]=S.coefVar;
    }           
    data.BPDLacAtGridStats= new FLstatistics (data.BPDLacAtGrid,
            grids);
}            

void InitializeEBPD()
{
    data.EMeanOfBPDAtEpsilon   =new float [grids][];
    data.EBPDLacAtEpsilon=     new float [grids][];
    data.EcvsqBPDLacAtEpsilon=new float[grids][];
    data.EStdDevOfBPDAtEpsilon=new float [grids][];        
    data.EVariationInBPDLacAtGrid=new float [grids];
    data.EBPDLacAtGrid=new float [grids];   
}
void InitializeEBPDAtEpsilon(int g, int e)
{
    data.EMeanOfBPDAtEpsilon[g]=new float [e];
    data.EBPDLacAtEpsilon[g]=new float [e];
    data.EcvsqBPDLacAtEpsilon[g]=new float [e];
    data.EStdDevOfBPDAtEpsilon[g]=new float [e];    
}

public void EBinnedProbabilityLacunarity()
{          
    InitializeEBPD();    
    for (int g=0; g<grids; g++)
    {
        InitializeEBPDAtEpsilon(g, F.Epsilons[g].length);
        for (int b = 0; b < F.Epsilons[g].length; b++)
        {  
            //bins at 0; probabilities at 1 binnedmprob holds 
            //an array of the probabilities sorted into categories
            //instead of foreach individual value so it has two arrays, 
            //one for the value and one for the frequency of that value
            float[][] binnedmprob =
               FLstatistics.binnedProbabilityDistribution(
                  AddZerosToArray(F.Pix[g][b],
                  F.Pix[g][b].length,(int)(F.Pix[g][b].length+
                  F.Empties[g][b])),
                  F.Pix[g][b].length+F.Empties.length,
                  V.MaxFrequencies, true);//V.isGray());
            float [] binnedPstuff =
                FLutil.BinnedProbabilityStats (binnedmprob, g);
            if (binnedPstuff==null)
                binnedPstuff=FLutil.newArray (4, 0.0f);
            data.EMeanOfBPDAtEpsilon[g][b]=binnedPstuff[0];
            data.EStdDevOfBPDAtEpsilon[g][b] =binnedPstuff[1];
            data.EcvsqBPDLacAtEpsilon[g] [b] =binnedPstuff[4];
            data.EBPDLacAtEpsilon[g][b] =binnedPstuff[3];
        }
    }
    for (int g=0; g<grids; g++)
    {//make arrays that summarizes the pd statistics
       FLstatistics S = new FLstatistics(data.EBPDLacAtEpsilon[g],
               data.EBPDLacAtEpsilon[g].length);
       data.EBPDLacAtGrid[g]=S.mean;     
       data.EVariationInBPDLacAtGrid[g]=S.coefVar;             
    }
    data.EBPDLacAtGridStats=
               new FLstatistics(data.EBPDLacAtGrid, grids);
}            
   
      
/**Makes a string of data for writing
* all of the box masses that were measured then
* displays it in a new text window for 
 * each {@link FLVars#GridPositions}.    */
public void recordRawData ()
{
        //Makes the text window's title according to 
        //the type of image and scan
        String S=V.isGray()?"Grayscale-":"Binary ";
        S=S+((V.isMF()) ?"Multifractal ":
             V.SLAC?"Overlapping ":"Nonoverlapping " ) ;
        //Makes a tabbed string listing all F.BOXSIZES
        String sizes = "";         
        for (int LOC = 0; LOC < V.GridPositions; LOC++)
        {
               //makes a separate window for each location                  
               sizes = "";
               for (int e=0; e <F.BOXSIZES[LOC].length; e++)        
                sizes=sizes+F.BOXSIZES[LOC][e]+tab;
                String masses="";           
                double longestlist=FLutil.LongestArray (F.Pix[LOC]);                     
                //the longest list determines rows (2 masses= 2 rows) 
                String RowOfMasses="";
                IJ.showStatus ("Writing box masses for location " +LOC);
                for (int row=0; row<longestlist; row++)
                {//on the first run, masses gets an empty line
                    //on subsequent runs,add a row onto the string of masses
                     //masses=masses+newline + RowOfMasses;
                     RowOfMasses="";
                     for (int b =0; b < F.BOXSIZES[LOC].length; b++)
                     {//if the list at this column is longer than the row, add
                         //the element at this column plus a tab  otherwise 
                         //add a tab only
                         if(F.Pix[LOC][b].length>row)
                             RowOfMasses=RowOfMasses+
                                     F.Pix[LOC][b][row]+tab;
                         else RowOfMasses=RowOfMasses+" "+tab ;                                 
                     }
                     masses=masses+newline + RowOfMasses;
                    //check every box size
                }//do a row for every element in the longest box size list
                TextWindow tw= new TextWindow ("Masses at "+LOC+
                     " ("+S+")"+F.Titles[LOC], sizes, masses, 500, 500);
        }
}
 }