// Notice
//     
// The contents of this file are subject to the
//  same license as the source code for ImageJ.  
//FracLac is in the public domain.  
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;
import ij.gui.*; 
import FLUtilities.FLutil;
import java.awt.Color;
    /** 
     * Contains methods to plot data from fractal, lacunarity,
     * and multifractal analyses.  Uses 
     */
    public class FLPlotter {//class to plot and compare data arrays
        
      
        /**Constructor accepts no parameters.   */        
        public FLPlotter()
        {}
  
        /**
         *
        
         */
    public static  void plotLogAndRegression(
            float []Y,
            float [] X,
            int N, 
            float Df, 
            float Yint, 
            float RSQ,        
            String title,
            String xaxis,
            String yaxis,
            boolean Log, 
            boolean RegressionLine) 
    {
        float[] RX = new float [N];
        float[] RY = new float [N];
        float [] LnY = null;  
        float [] LnX = null;
        
        if (Log) {
            LnX=FLutil.LogArray(X, N);
            LnY=FLutil.LogArray(Y, N);
        }
        float xmin=LnX[0], xmax=LnX[0], ymin=LnY[0], ymax=LnY[0];
        
        for (int i = 0; i< N; i++) {
          if(RegressionLine)
          {
                RX[i] = LnX[i];
                RY[i] = -Df*RX[i]+Yint; 
          }           
            xmin = FLutil.min(xmin, RX[i], LnX[i]);
            ymin = FLutil.min(ymin, RY[i], LnY[i]);
            xmax = FLutil.max(xmax, RX[i], LnX[i]);
            ymax = FLutil.max(ymax, RY[i], LnY[i]);
        }

        
        PlotWindow pl = new PlotWindow(title, xaxis, yaxis, X, Y);      
        pl.setColor(Color.red.darker());
        pl.addPoints(X, Y,PlotWindow.CIRCLE);
        pl.addLabel(.2,.3, title);
        pl.draw();

        if(Log||RegressionLine)
        { 
            PlotWindow plot = new PlotWindow(title,
                    (Log?"Ln ":"")+xaxis, 
                (Log?"Ln ":"")+yaxis, LnX, LnY); 
        if(Log){plot.setColor(Color.blue.darker());
        plot.addPoints(LnX, LnY,PlotWindow.CIRCLE);
        plot.addLabel(.2,.3, title);
        plot.draw();
        }
        if(RegressionLine)
        {plot.setColor(Color.green);
        plot.addLabel(1/2, 1/2, "Slope = "
        +FLutil.fnum(Df)+FLMain.rsq+"="+FLutil.fnum(RSQ));          
        plot.setLineWidth(1);
        plot.addPoints(RX, RY, PlotWindow.LINE);
        plot.draw();
        }}     
       
    }
    
    
    
    

    
    /**
     * Shows graphs of previously prepared
     * multifractal spectra data from
     * box counting, returning no value.
     * <h3>Note:</h3>
     * <ul>
     * <li>
     * Multifractal analysis exaggerates differences
     * in scaling by raising the distribution of pixels per box
     * to a set of exponents, Q, and recalculating fractal
     * measures.</li><li>The multifractal spectra graphically
     * display this mathematical distortion.</li>
     * <li>A typical &#402;(&#945;) vs &alpha; 
     * multifractal spectrum is convex,
     * humped upward (at least for Q>0) and diverging.</li>
     * <li>A monofractal spectrum, in contrast, converges.</li>
     * <li>The D<sub>Q</sub> at Q=0
     * corresponds to the box counting dimension.</li>
     * <li>The
     * plot of D<sub>Q</sub> vs Q is generally decreasing for
     * multifractals but not monofractals.</li>
     * <li>Box counting sampling may not represent the
     * probability distribution, so the graphs may deviate somewhat,
     * especially for Q<0 where the distortion of very small
     * probabilities is prominent.  The correction employed here
     * is to improve sampling using controlled
     * runSubs random mass samples,
     * and, more effectively, shifting 
     * {@link FLVars#GridPositions the grid}.
     * </ul>
     *
     * @see     
     * FLAnalyzer.FLMain#FinishQslopes 
                preparatory method
     * @param   Qs float [] for Qs (horizontal axis with.  Plots
     *          positive and negative values with different colours.
     * @param   GraphFA boolean switch
     *   for graphing &#402;(&#945;)
     * @param   alphaz float [] of &#945;
     * @param   Fatalphaz float [] of &#402;(&#945;)
     * @param   LOC int for location
     * @param   imgtitle String for the title of the graph, based
     *          on the image is represents
     */
    public static void PlotFAndAlpha(
    float []alphaz,
    float[] Fatalphaz,
    int LOC,
    String imgtitle,
    float [] Qs,
    boolean GraphFA) {
        {
            float amax = alphaz[0];
            float amin = amax;
            float fmax = Fatalphaz[0];
            float fmin = fmax;
            float dqat0=0;
            int negs = 0;
            int pos = 0;
            for (int c = 0; c < Qs.length; c++) {
                if (Qs[c]<=0)negs++;
                if(Qs[c]>=0)pos++;
                if (Qs[c]==0)dqat0=Fatalphaz[c];
            }
            float [] pA = FLutil.newArray(pos, 0.0f);
            float [] nF = FLutil.newArray(negs, 0.0f);
            float [] nA = FLutil.newArray(negs, 0.0f);
            float [] pF = FLutil.newArray(pos, 0.0f);
            int n = 0, a = 0;
            for (int u=0; u <Qs.length; u++) {
                
                if(Qs[u]>=0) {
                    pA[a]=alphaz[u];
                    pF[a]=Fatalphaz[u];
                    a++;
                }
                if(Qs[u]<=0) {
                    nA[n]=alphaz[u];
                    nF[n]=Fatalphaz[u];
                    n++;
                }
            }
            
            for (int q = 0; q < Qs.length; q++) {               
                amin = Math.min(alphaz[q], amin);
                amax = Math.max(alphaz[q], amax);
                fmax = Math.max(Fatalphaz[q], fmax);
                fmin = Math.min(Fatalphaz[q], fmin);
            }
            float line = fmax;          
                 
            amin=0;
            amax=3;
            fmin = 0;
            fmax = 2;
            float []x =
            {amin, amax};
            float []y=
            {fmax, fmax};
            String title = imgtitle+" # "+ LOC +" Q("
            + FLutil.fnum(Qs[0])
            + " to "
            + FLutil.fnum(Qs[Qs.length-1])+")";
            if (GraphFA) {
                PlotWindow pmf = new PlotWindow( title, 
                        FLMain.alpha,
                "F(" +
                       FLMain.alpha +
                        ")", x, y);
                pmf.setLimits(amin, amax, fmin, fmax);
                pmf.addLabel(.2, .1, "max="+FLutil.fnum(line));
                pmf.addLabel(.2, .2, "q=0: "+FLutil.fnum(dqat0));
                pmf.draw();
                pmf.setLineWidth(1);
                pmf.setColor(Color.green);
                pmf.addPoints(pA, pF, PlotWindow.CROSS);
                
                pmf.addLabel(.2, .4, "Q>=0: GREEN X");
                pmf.setLineWidth(1);
                pmf.setColor(Color.red);
                pmf.addPoints(nA, nF,  PlotWindow.CIRCLE);
                pmf.addLabel(.7, .4, "Q<=0: RED O");
                pmf.draw();
            }            
        }
    }
        
    }
    
    