// Notice
// The contents of this file are subject to the
//  same license as the source code for ImageJ.
// FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;

import ij.IJ;
import ij.gui.GenericDialog;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**Contains classes for getting user inputs and setting up the results
 * table.*/
public class FLsetup
{
    
    /** Creates a new instance of flsetup */
    public FLsetup ()
    {    }
    public static String [] S ={ "INCLUDE CIRCULARITY DATA:",
            "DRAW HULL","DRAW CIRCLE"},
             BGFG={"Let the program choose", "Black", "White"};
    public static String minsize =
            "Minimum size for scaled, relative, and linear series",
            seriestype ="Type of Series", usegreater= "Use greater" +
                         " dimension of ROI", 
            maxbox="Maximum box size", 
            override="pixels: overrides % if >0",
            numsizes="Sizes per series (may be overridden if invalid):" +
                         "(0 calculates automatically; )", 
            numgrids="Number of grid positions:",
            block = "Block (texture/filled)",
            printfreq="Print Mass vs Frequency Distributions",
            printraw="Print raw data (box masses)",
            bins="Bins for Frequency Distributions",
            bg="Background Colour",
            defsize="Use Default Box Sizes",
            pow="Use Power Series",
            scaled="Use Scaled Series",
            relative="Use Relative Sizes",
            bingray="Binary or Grayscale",
            pixcheck ="Check Pixel Ratio";
/**Returns an array of values specified by the passed values.  The
* values are limited:<ul><li>if (Math.abs(QS[i]-1.00000)
* <0.00001)QS[i]=1</li><li>if (Math.abs(QS[i]-0.00000)
* <0.00001)QS[i]=0</il><ul> In FracLac, the returned array
* of values is Qs for calculating the generalized dimension 
* spectrum, based on user inputs for the minimum, maximum, 
* and increment between values.
* @return float [] from QMIN to QMAX, separated by QINC     
* @param qinc float for the increment between Qs
* @param QMIN float for the minimum value in the array
* @param QMAX float for the maximum value in the new array
* @param QINC float for the increment
* between values in the new array*/
public static float [] MakeAnArrayOfQs (
    float QMIN, float QMAX, float QINC, float qinc)
{
        float S = ((QMAX-QMIN)+1)/qinc;
//  the number of Qs to make is 1+ the
    int SIZE=(int)S;                  //difference between the min and max,
        //                                 divided by the increment
        
        float [] QS= new float [SIZE];
        
        float qq=QMIN;
        
        //                                  loop to make an array of Qs from
        //                                  the minimum to the maximum
        //                                  with the increment between values
        
        for (int i = 0; (i<SIZE); i++)
        {
            QS[i]=qq;
            if (Math.abs (QS[i]-1.00000)<0.00001)QS[i]=1;
            if (Math.abs (QS[i]-0.00000)<0.00001)QS[i]=0;
            qq=qq+QINC;
        }
        
        return QS;
    }
    
    static String bingraythresh []={"Use Binary",
                "AutoThreshold to Binary", "Use Grayscale"};
      
/**Gets user inputs for box counting scans 
* from a popup menu then sets variables in the
* class's instance of a {@link FLAnalyzer.FLVars variable holder}.    
* @return FLVars with new inputs
* @see FLAnalyzer.FLDataProcessor#getBinarystats
* @see FLAnalyzer.FLMain#BoxCountDataWriter
* @param f FLVars instance of variables to assess and change
* @throws NumberFormatException if number of origins is bad
*/
public static FLVars getBCInputs (FLVars f) 
throws NumberFormatException
{            
    GenericDialog gd
    = new GenericDialog ("FracLac Options", IJ.getInstance ());            
    gd.addChoice (bingray,  
            bingraythresh, bingraythresh[f.getImageType ()] );
    String CS[] =  {pixcheck,  block,  "Show Grids"};
    boolean[] CD =  { f.CheckPixRatio,
            f.BLOCKTEXTUREANALYSIS,  f.showGrids};
    gd.addCheckboxGroup (1, CD.length, CS, CD);     
    gd.addChoice (bg, BGFG, BGFG[f.ChooseBackground]);
    String sizes[]=new String [4];
    sizes[f.DEFAULTBOXSIZES]=defsize;
    sizes[f.POWERSERIES]= pow;
    sizes[f.SCALEDSERIES]= scaled;
    sizes[f.RELATIVESERIES]=relative;
    gd.addChoice (seriestype, sizes,   sizes[f.Series]);
    gd.addSlider (numsizes, 0.0,255.0, f.getMaxNumBoxSizes());          
    gd.addNumericField(minsize,
            f.getUserMinBoxSize (),0,3,"pixels");
    gd.addNumericField (maxbox, f.maxpercent, 0, 4,"% of ROI");
    gd.addCheckbox (usegreater, f.UseGreater);           
    gd.addNumericField (maxbox, f.getUserMaxBoxSize(),
            0, 4,  override);
    JTextField Field = scrollingNumberField ( f.GridPositions, 1);
    gd.add ( new JLabel (numgrids));
    gd.add ( Field);
    gd.addCheckbox ("Show Data", f.ShowData);
    gd.addCheckbox (printraw,  f.WriteRawData);
    gd.addCheckbox ("Slip grid at each epsilon:", f.slipGrid);
    gd.addCheckbox (printfreq,f.printFrequencies);
    gd.addNumericField (bins, f.MaxFrequencies, 0);            
    gd.addCheckbox ("GRAPH REGRESSION" +
        " LINE FOR EACH SAMPLE",  f.isGraphRegression ());
    gd.addCheckbox ("GRAPH Lacunarity",  f.isGraphLac ());           
    gd.addCheckboxGroup (1, 3, S, new boolean [] { f.isDocircul (),
        f.DRAWHull, f.DRAWCircle});            
    gd.showDialog ();
    if (gd.wasCanceled ())
    {f.proceed=false; return f;}
    else
    {       
        f.setUseParticleAnalyzer (false);
        f.showcolours=false;                        
        f.UseRectAnalyzer=false;
        f.ScanSeparateRandomSubAreas=false;
        f.MultiFiles=true;
        f.BottomRight=false;
        f.SLAC=false;                  
        f.setMF (false);               
        f.setImageType (gd.getNextChoiceIndex ());
        f.CheckPixRatio=gd.getNextBoolean ();
        f.BLOCKTEXTUREANALYSIS= gd.getNextBoolean ();
        f.showGrids=gd.getNextBoolean ();
        f.ChooseBackground= gd.getNextChoiceIndex ();
        if(f.ChooseBackground==f.USEBLACK)
        {
            f.background=f.BLACK;
            f.foreground=f.WHITE;
        }
        if(f.ChooseBackground==f.USEWHITE)
        {
            f.background=f.WHITE;
            f.foreground=f.BLACK;
        }
        f.Series= gd.getNextChoiceIndex ();                   
        f.setMaxNumBoxSizes ( (int)gd.getNextNumber ());
        if(f.getMaxNumBoxSizes ()<=0)
        {
            f.setMaxNumBoxSizes (0);
            f.setUserWantsToSpecifyNumberOfBoxes (false);
        }
        else f.setUserWantsToSpecifyNumberOfBoxes (true);
        f.setUserMinBoxSize ((int)gd.getNextNumber ());
        if (f.getUserMinBoxSize ()<=1) f.setUserMinBoxSize (1);
        f.maxpercent=  (float)gd.getNextNumber ();
        if(f.maxpercent<=1){ f.maxpercent=1;}
        f.UseGreater=gd.getNextBoolean ();    
        f.setUserMaxBoxSize ( (int)gd.getNextNumber ());
        try
        {
            f.GridPositions = (int)java.lang.Double.parseDouble (
                    Field.getText ());
        }
        catch (NumberFormatException nf){f.GridPositions=1;}
        f.ShowData = gd.getNextBoolean ();
        f.WriteRawData = gd.getNextBoolean ();
        f.slipGrid=gd.getNextBoolean ();
        f.printFrequencies = gd.getNextBoolean ();
        f.MaxFrequencies = (int)gd.getNextNumber ();
        if (f.MaxFrequencies<5)f.MaxFrequencies = 5;                    
        f.setGraphRegression (gd.getNextBoolean ());
        f.setGraphLac(gd.getNextBoolean());
        f.setDocircul (gd.getNextBoolean ());
        f.DRAWHull=gd.getNextBoolean ();
        f.DRAWCircle=gd.getNextBoolean ();
        if (f.DRAWCircle||f.DRAWHull)f.setDocircul (true);
        if ((f.GridPositions <= 0)){ f.GridPositions = 0;}                                                 
        f.Globalxy= new int[f.GridPositions][2];
        for (int i = 0; i < f.GridPositions;i++)
        {
            f.Globalxy[i][0]=0;
            f.Globalxy[i][1]=0;
        }
            f.proceed=true;
            if(f.getImageType ()==f.GRAY)
            {
                f.setDocircul (false);
                f.DRAWHull=false;
                f.DRAWCircle=false;
                }
        if (f.isGray ()) f.grayscantype=set2DGray (f.grayscantype);
        if(f.GraphLac)f.SameScale=setSameScale();
        if(f.CheckPixRatio)f.FACTOR=getFactor (f.FACTOR);                        
        //decide how to then calculate the box sizes
        if(f.Series==f.POWERSERIES)
            f = getPowerSeriesOptions (f);
        else if (f.Series==f.SCALEDSERIES)
            f=FLsetup.GetScaledSeries (f);
        else if (f.Series==f.RELATIVESERIES)
            f=  showRelativeMessage(f);
        return f;
    }
}
    


static FLVars  showRelativeMessage(FLVars f)
  {
    int m=f.getUserMaxBoxSize ();
    m=(int)IJ.getNumber("Relative sizes are " + 
              " factors of MAX " +
              " (e.g. 6=1,2,3,6 " +
              "but 7 = 1,7).  0 finds Max or Max=",m);
      if (m<1)m=0;
      f.setUserMaxBoxSize(m);
      return f;
  }  
    
    
    

/** Makes a number field that scrolls up and down with mouse clicks.
* @return javax.swing.JTextField
* @param numberofgrids int for the
* @param defaultnum to reset field in case of trouble
* @throws NumberFormatException when parsing the field*/
public static JTextField  scrollingNumberField (int numberofgrids,
        final int defaultnum) throws NumberFormatException
{
    final JTextField field
            = new JTextField (Integer.toString (numberofgrids));
    field.setSize (30, 30);
    field.addMouseListener (new MouseAdapter ()
    {
        public void mousePressed (MouseEvent e)
        {
            int change = 1;
            if(SwingUtilities.isRightMouseButton (e))change=-1;
            int num = defaultnum;
            boolean ex= false;
            try
            {
                num = (int)java.lang.Double.parseDouble (
                        field.getText ());
            }
            catch (NumberFormatException nf)
            {ex=true;}
            finally
            {
                if (ex== true)
                {field.setText (Integer.toString (defaultnum));}
            }
            num = num + change;
            if (num<0)num=1;
            field.setText (Integer.toString (num));                
        }
    });
    return field;
}
     
    
/**Gets the settings from the user to do a {@link 
 * FLAnalyzer.FLMain#GetSLACData }sliding box lacunarity
 * algorithm.
* @return FLVars with the new settings
* @param f {@link FLAnalyzer.FLVars} */
public static FLVars getSLacInputs (FLVars f)
{
    GenericDialog gd = new GenericDialog
             ("FracLac Sliding Box Options", IJ.getInstance ());                 
    String S[] =  { block, "Show Grids",   "Scan Edges", pixcheck};
    boolean[] D={f.BLOCKTEXTUREANALYSIS,
            f.showGrids, f.ScanEdges, f.CheckPixRatio}; 
    gd.addChoice (bingray, bingraythresh, 
            bingraythresh[f.getImageType()] );
    gd.addChoice (bg, BGFG, BGFG[f.ChooseBackground]);
    gd.addCheckboxGroup (1, D.length, S, D);
    String sizes[]=new String [4];
    sizes[f.DEFAULTBOXSIZES]=defsize;
    sizes[f.POWERSERIES]= pow;
    sizes[f.SCALEDSERIES]= scaled;
    sizes[f.RELATIVESERIES]=relative;
    gd.addChoice (seriestype,   sizes,  sizes[f.Series]);
    gd.addNumericField( numsizes,  f.getMaxNumBoxSizes(), 0);
    gd.addNumericField(minsize,f.getUserMinBoxSize (),0,1,"pixels");
    gd.addNumericField(maxbox, f.maxpercent, 2, 1, "% of ROI");
    gd.addCheckbox (usegreater, f.UseGreater);
    gd.addCheckbox("Show raw data?", f.WriteRawData);
    gd.addNumericField(maxbox,f.getUserMaxBoxSize(),0,1,override);
    gd.addNumericField("Number of pixels to slide boxes (X axis)",
             f.SLIDEX, 0);
    gd.addNumericField("Number of pixels to slide boxes (Y axis)",
             f.SLIDEY, 0);      
    gd.addNumericField (bins, f.MaxFrequencies, 0);
    gd.addCheckbox ("Show graphs", f.isGraphLac ());
    gd.addCheckbox ("Write box masses", f.WriteRawData);
    gd.showDialog ();
    if (gd.wasCanceled ())
     {
         f.proceed=false;
         return f;
     }
    else
    {
         f.setMF(false);
         f.DRAWCircle=false;
         f.DRAWHull=false;                     
         f.BottomRight=false;                   
         f.SLAC=true;                     
         f.BottomRight=false;        
         {
                f.setUseParticleAnalyzer (false);
                f.showcolours=false;                            
                f.UseRectAnalyzer=false;
                f.ScanSeparateRandomSubAreas=false;
         }                    
         f.BLOCKTEXTUREANALYSIS=gd.getNextBoolean ();
         f.showGrids=gd.getNextBoolean ();
         f.ScanEdges=gd.getNextBoolean();          
         f.CheckPixRatio=gd.getNextBoolean();
         f.setImageType(gd.getNextChoiceIndex ());                                           
         f.ChooseBackground=gd.getNextChoiceIndex ();
         if(f.ChooseBackground==f.USEBLACK)
         { f.background=f.BLACK; f.foreground=f.WHITE;}
         if(f.ChooseBackground==f.USEWHITE)
         { f.background=f.WHITE; f.foreground=f.BLACK;}                     
         f.Series=(gd.getNextChoiceIndex ());                   
         f.setMaxNumBoxSizes((int)gd.getNextNumber ());
         if(f.getMaxNumBoxSizes()<=0)
         {
             f.setMaxNumBoxSizes(0);
             f.setUserWantsToSpecifyNumberOfBoxes (false);
         }
         else f.setUserWantsToSpecifyNumberOfBoxes (true);                                                   
         f.setUserMinBoxSize ((int)gd.getNextNumber ());
         if (f.getUserMinBoxSize ()<=1) f.setUserMinBoxSize(1);                                          
         f.maxpercent=(float)gd.getNextNumber ();
         if(f.maxpercent<=1) f.maxpercent=1;                                         
         f.UseGreater=gd.getNextBoolean ();
         f.WriteRawData=gd.getNextBoolean();
         f.setUserMaxBoxSize((int)gd.getNextNumber ());                  
         f.GridPositions=1;                              
         f.RandomMassSample=false;                   
         f.Globalxy=new int[f.GridPositions][2];                     
         for (int i = 0; i < f.GridPositions;i++)
         {
             f.Globalxy[i][0]=0;
             f.Globalxy[i][1]=0;
         }                     
         f.SLAC = true;
         f.SLIDEX=(int)gd.getNextNumber ();
         f.SLIDEY=(int)gd.getNextNumber ();
         f.MaxFrequencies=(int)gd.getNextNumber ();
         if(f.MaxFrequencies<2)f.MaxFrequencies=2;
         f.setGraphLac (gd.getNextBoolean ());
         f.WriteRawData=gd.getNextBoolean ();
         f.proceed=true;
         if (f.isGray ()) f.grayscantype=set2DGray (f.grayscantype);
        if(f.GraphLac)f.SameScale=setSameScale();
        if(f.Series==f.POWERSERIES)                    
            f = FLsetup.getPowerSeriesOptions (f);                    
        else if (f.Series==f.SCALEDSERIES)
            f=FLsetup.GetScaledSeries (f);
        else if (f.Series==f.RELATIVESERIES)
            f=  showRelativeMessage(f);
        if(f.CheckPixRatio)f.FACTOR=getFactor (f.FACTOR);     
        return f;
     }
 }


/**Gets the user's inputs to do a multifractal analysis.  Note that 
* doing this changes how grid locations are located.Note that this
* sets {@link FLAnalyzer.FLVars#BottomRight BottomRight} to 
* true, so that a specialized multifractal partitioning is used which 
* means that the grid is placed at each of the four corners for the 
* first four scans, rather than shifted around a point.
* @see FLAnalyzer.FLVars#BottomRight
* @return boolean always true when done
* @param f {@link FLAnalyzer.FLVars}
* @see FLAnalyzer.FLMain#MultifractalDataProcessor multifractal 
* data processor  */
public static FLVars getMFInputs (FLVars f)
{                         
    String [] mass={"None",  "Random Mass Sample"};                 
    GenericDialog gd = new GenericDialog
    ("Multifractal Analysis Options",  IJ.getInstance ());                 
    String CS[] = {pixcheck, "Show Grids"};
    gd.addChoice ("Use Binary - Threshold - Use Grayscale",
            bingraythresh, bingraythresh[f.getImageType()] );
    gd.addChoice ("Data Processing", f.DS, f.DS[f.dataoption]);
    gd.addChoice (bg,  BGFG,  BGFG[f.ChooseBackground]);
    boolean[] CD = {f.CheckPixRatio, f.showGrids};
    gd.addCheckboxGroup (1, CD.length, CS, CD);                         
    String sizes[]=new String [4];
    sizes[f.DEFAULTBOXSIZES]=defsize;
    sizes[f.POWERSERIES]= pow;
    sizes[f.SCALEDSERIES]= scaled;
    sizes[f.RELATIVESERIES]=relative;
    gd.addChoice(seriestype, sizes, sizes[f.Series]);
    gd.addNumericField(numsizes ,f.getMaxNumBoxSizes(), 0);
    gd.addNumericField(minsize,f.getUserMinBoxSize(),0,1,"pixels");
    gd.addNumericField (maxbox,f.maxpercent, 2, 1, "% of ROI");
    gd.addCheckbox (usegreater, f.UseGreater);
    gd.addNumericField(maxbox,f.getUserMaxBoxSize(),0,1,override);                         
    JTextField Field = scrollingNumberField ( f.GridPositions, 5);
    gd.add ( new JLabel (numgrids));
    gd.add ( Field);                         
    String S[] = {"Show data", "Include all box masses",
            "Probability Distributions",  "Slip grid at epsilon",};
    boolean[] D = {f.ShowData, f.WriteRawData, 
            f.printFrequencies, f.slipGrid};
    gd.addCheckboxGroup (1, D.length, S, D);
    gd.addMessage ("Show Graphs:");
    String [] labels ={"D(Q) vs Q", FLMain.alpha+" vs Q", 
            FLMain.falpha+" vs Q", FLMain.falpha+" vs "+FLMain.alpha,
            "Regression", FLMain.tau, FLMain.Lambda};
    boolean [] defaults = {f.isGraphQs (),  f.isGraphAQ (),
            f.isGraphFQ (),  f.isGraphFA (),  f.isGraphRegression (),
            f.isGraphT (),  f.GraphLac};
    gd.addCheckboxGroup(1,defaults.length,labels,defaults);
    gd.addCheckbox("Show Highest CV",f.SelectBestCV);
    gd.addCheckbox ("Print Slope and CV for D(Q)vsQ",
            f.WriteQslopeandCV);
    gd.addNumericField (bins, f.MaxFrequencies, 0);
    gd.addNumericField ("Maximum Q for Generalized " +
            "Dimension Spectrum:",  f.getQmax (), 2);
    gd.addNumericField("Minimum Q:",f.getQmin (),2);
    gd.addNumericField  ("Increment between Qs:", f.getQinc (), 3);
    int b= f.RandomMassSample?1:0;
    gd.addChoice ("Select mass method",  mass, mass[b]);
    gd.addNumericField ( "Size of sample (% of image size)", 
            f.percentsubboxsize, 0);
    gd.addNumericField ("Maximum grid (% of sample)",
            f.RandomMassMaxSubpercent, 0);
    gd.addNumericField ("Number of samples:", f.subsamples, 0);
    gd.addNumericField ("Minimum pixel ratio", f.getFACTOR (), 4);                                           

   gd.showDialog ();
   if (gd.wasCanceled ())
   {
       f.proceed=false;
       return f;
   }
   else
   {                
        f.BLOCKTEXTUREANALYSIS=false;  
        f.BottomRight=true;
        f.setMF(true);
        f.SLAC=false;    
        f.DRAWHull=false;
        f.DRAWCircle=false;
        f.setDocircul (false);                      
        f.setUseParticleAnalyzer (false);
        f.UseRectAnalyzer=false;
        f.ScanSeparateRandomSubAreas=false;
        f.setImageType(gd.getNextChoiceIndex ());   
        f.dataoption=gd.getNextChoiceIndex ();
        f.ChooseBackground= gd.getNextChoiceIndex ();
        if(f.ChooseBackground==f.USEBLACK)
        {
            f.background=f.BLACK;
            f.foreground=f.WHITE;
        }
        if(f.ChooseBackground==f.USEWHITE)
        { 
            f.background=f.WHITE;
            f.foreground=f.BLACK;
        }
        f.CheckPixRatio = gd.getNextBoolean ();  
        f.showGrids=gd.getNextBoolean ();
        f.Series=  gd.getNextChoiceIndex ();                           
        f.setMaxNumBoxSizes( (int)gd.getNextNumber ());
        if(f.getMaxNumBoxSizes()<=0)
        {
             f.setMaxNumBoxSizes(0);
             f.setUserWantsToSpecifyNumberOfBoxes(false);
        }
        else f. setUserWantsToSpecifyNumberOfBoxes (true);
        f.setUserMinBoxSize ( (int)gd.getNextNumber ());
        if (f.getUserMinBoxSize()<=1) f.setUserMinBoxSize(1);
        f.maxpercent= (float)gd.getNextNumber ();
        if(f.maxpercent<=1) { f.maxpercent=1;}
        f.UseGreater=gd.getNextBoolean ();
        f.setUserMaxBoxSize( (int)gd.getNextNumber ());
        try
        {
           f.GridPositions = ((int)java.lang.Double.parseDouble (
                       Field.getText ()));
        }
       catch (NumberFormatException nf){ f.GridPositions =(1);}
       if ((f.GridPositions  <= 0)) { f.GridPositions =(1); }
       f.ShowData = gd.getNextBoolean ();
       f.WriteRawData = gd.getNextBoolean ();
       f.printFrequencies = gd.getNextBoolean ();
       f.slipGrid=gd.getNextBoolean ();
       f.setGraphQs (gd.getNextBoolean ());
       f.setGraphAQ (gd.getNextBoolean ());
       f.setGraphFQ (gd.getNextBoolean ());
       f.setGraphFA (gd.getNextBoolean ());
       f.setGraphRegression (gd.getNextBoolean ());
       f.setGraphT (gd.getNextBoolean ());
       f.GraphLac=gd.getNextBoolean();
       f.SelectBestCV=gd.getNextBoolean ();
       f.WriteQslopeandCV=gd.getNextBoolean ();
       f.MaxFrequencies =  (int)gd.getNextNumber ();
       if (f.MaxFrequencies<2)  f.MaxFrequencies = 2;
       f.setQmax ((float)gd.getNextNumber ());
       f.setQmin ((float)gd.getNextNumber ());
       f.setQinc ((float)gd.getNextNumber ());
       if (f.getQinc ()<0)f.setQinc (f.getQinc ()*-1.0f);
       f.Qs=FLsetup.MakeAnArrayOfQs (f.getQmin (), f.getQmax (),
               f.getQinc (), f.getQinc ());
       f.percentsubboxsize = (float) gd.getNextNumber ();
       if(f.percentsubboxsize<.5f) f.percentsubboxsize=.5f;
       f.RandomMassMaxSubpercent=(float)gd.getNextNumber();
       String masstype=gd.getNextChoice ();                                                                                  
       f.RandomMassSample=false;                           
       if (masstype==mass[1]) f.RandomMassSample=true;
       if(f.RandomMassSample)
        {                                         
            f.MultiFiles=false;
            f.GridPositions=1;                       
            if (f.dataoption==0|| f.dataoption==1);
            else f.dataoption=0;
        }
        f.Globalxy=new int[f.GridPositions][2];
        for (int i = 0; i  < f.GridPositions;i++)
        {
            f.Globalxy[i][0]=0;
            f.Globalxy[i][1]=0;
        }
        f.subsamples=(int)gd.getNextNumber ();
        if(f.subsamples<=0) f.subsamples=1;
        f.setFACTOR ((float)gd.getNextNumber ());
        if (f.getFACTOR ()<=0) f.setFACTOR (0);
        f.proceed=true;
        if (f.isGray ()) f.grayscantype=set2DGray (f.grayscantype);
        if(f.GraphLac)f.SameScale=setSameScale();
        if(f.Series==f.POWERSERIES)
            f = FLsetup.getPowerSeriesOptions (f);                
        else if (f.Series==f.SCALEDSERIES)
            f=FLsetup.GetScaledSeries (f);
        else if (f.Series==f.RELATIVESERIES)
            f=  showRelativeMessage(f);
       return f;        
   }
}

/**Array of strings specifying the types of scans.*/
public static String []SubScanTypes=
{
                    "Analyzer Scan-contours",
                    "Analyzer Scan-filled",
                    "Analyzer Scan-display dimension",

                    "Rectangular Scan-display dimension",
                    "Rectangular Scan-blocks",
                    "Rectangular Scan-pixels",

                    "Random Block Scan-filled",
                    "Random Block Scan-colour pixels",
                    // "Block Scan-no colour",
                    "Random Block Scan-no colour",
                    //"Block Scan-colour pixels",
                    //"Block Scan-fill",
                         
};

/**Shows a GenericDialog to get the base and increase between 
* exponents for a series of exponential increases.  If the dialog is not
* cancelled, the value of FLVars.base is changed to the first number, 
* and FLVars.exponent is changed to the second number requested
* in the dialog. {@link FLAnalyzer.FLVars#exponent} is changed to
* false if the dialog is cancelled; otherwise it is changed to true. The
* minimum for the base is 1.1; it is changed up to this if the user
* inputs a lower number. 
* @param f FLAnalyzer.FLVars instance of variables
* @return FLAnalyzer.FLVars the modified instance of stored
* variables, with {@link FLAnalyzer.FLVars#base}, and
* {@link FLAnalyzer.FLVars#exponent} potentially changed.*/
public static FLVars getPowerSeriesOptions (FLVars f)
  {
         GenericDialog gd= new GenericDialog
                 ("Series Options", IJ.getInstance ());
         gd.addNumericField (
                 "Base (minimum) for series of box sizes", f.base, 4);
         gd.addNumericField (
                 "Exponential increase for box sizes", f.exponent, 4);
         gd.showDialog ();
         if (gd.wasCanceled ())
         {
             f.Series=f.DEFAULTBOXSIZES;                     
             return f;
         }
         else
         {
             f.base=(float)gd.getNextNumber ();
             if (f.base<=1.00f)f.base=1.01f;
             f.exponent=(float)gd.getNextNumber ();
         }
         return f;                 
 }
             
/**Gets user inputs for making a scaled series of box sizes.
* @param f FLVars instance 
* @return FLVars instance of variables*/
public static FLVars GetScaledSeries (FLVars f)
{
    GenericDialog gd=new GenericDialog("Scale Options",
            IJ.getInstance ());
    gd.addNumericField ("Largest Box Size (0 chooses for you)",
         (double)f.ScaledMaxBoxSize, 0 );
    gd.addNumericField ("Scale for box sizes (numerator)", 
         (float)f.BoxSizeScaleNumerator, 2);
    gd.addNumericField ("Scale for box sizes (denominator)",
         (float)f.BoxSizeScaleDenominator, 2);
    gd.showDialog ();
    if (gd.wasCanceled ())
    {
        f.Series=f.DEFAULTBOXSIZES;
        return f;
    }
    else
    {
        f.ScaledMaxBoxSize=(float)gd.getNextNumber ();
        if(f.ScaledMaxBoxSize<0)
            f.ScaledMaxBoxSize=f.ScaledMaxBoxSize*-1f;
        f.BoxSizeScaleNumerator= (float)gd.getNextNumber ();
        if(f.BoxSizeScaleNumerator<=0)
            f.BoxSizeScaleNumerator= -1f*f.BoxSizeScaleNumerator;
        f.BoxSizeScaleDenominator= (float)gd.getNextNumber ();
        if(f.BoxSizeScaleDenominator<=0)
            f.BoxSizeScaleDenominator =-1f*f.BoxSizeScaleDenominator;                     
    }
    return f;
 }                  
           
/**Gets user's inputs for subscans.
* @param f {@link FLAnalyzer.FLVars}
* @return {@link FLAnalyzer.FLVars} */
 public static FLVars GetSubScanInputs (FLVars f)
{
     GenericDialog sd = new GenericDialog ("Sub Scan Options",
     IJ.getInstance ());           
//request scanning option     
     sd.addChoice ("Type of Scan", SubScanTypes, 
             SubScanTypes[f.SubScanType]);
//Request image type choice
     sd.addChoice (bingray, bingraythresh,
             bingraythresh[f.getImageType ()] );  
     String CS[] =  {pixcheck,  block, "Show Grids"};
     boolean[] CD={f.CheckPixRatio,f.BLOCKTEXTUREANALYSIS,
             f.showGrids}; 
     sd.addCheckboxGroup (1, CD.length, CS, CD);              
//Request background option                  
     sd.addChoice (bg,  BGFG, BGFG[f.ChooseBackground]);   
//Request type of box size series
     String sizes[]=new String [4];
     sizes[f.DEFAULTBOXSIZES]=defsize;
     sizes[f.POWERSERIES]= pow;
     sizes[f.SCALEDSERIES]= scaled;
     sizes[f.RELATIVESERIES]=relative;
     sd.addChoice ("Select a type of box size series", sizes,
             sizes[f.Series]);
//Request maximum number of boxes
     sd.addSlider("Box sizes per scan (0 automatically calculates)",
             1, f.absolutemaximumnumberofboxes,
             f.getMaxNumBoxSizes () );
     sd.addCheckbox ("Use greater ROI dimension for maximum " +
             "size",   f.UseGreater);  
//Request size of scan sample            
     sd.addNumericField ("Sample size", f.Subboxsize, 0, 4, "pixels");
     sd.addNumericField("Size of largest box relative to sample size", 
             f.maxpercent, 0, 4, "%");
     sd.addNumericField("Size of largest box",
             f.getUserMaxBoxSize(), 0, 4, override);
     sd.addNumericField ("Size of smallest box", 
             f.getUserMinBoxSize (),  0,3,"pixels");                       
//set data viewing options
     sd.addCheckbox  ("Show Data", f.ShowData);
     sd.addCheckbox  (printraw, f.WriteRawData);            
     sd.addCheckbox (printfreq, f.printFrequencies);
     sd.addNumericField (bins, f.MaxFrequencies, 0);            
//set graphing options         
     sd.addCheckbox ("GRAPH REGRESSION" +
             " LINE FOR EACH SAMPLE",   f.isGraphRegression ());
     sd.addCheckbox ("GRAPH Lacunarity",  f.isGraphLac ());            
//get hull and circle options            
     sd.addCheckboxGroup (1, 3, S, new boolean [] { f.isDocircul (),
             f.DRAWHull, f.DRAWCircle});
//=====         SHOW DIALOG AND GET INPUTS  =====//
     sd.showDialog ();
     if (sd.wasCanceled ())
     {
         f.proceed=false;
         return f;
     }
     else
     {//set nonnegotiable default values for sub scans 
         f.MultiFiles=true;
         f.BottomRight=false;
         f.SLAC=false;                  
         f.setMF (false);       
         f.slipGrid=false;       
         f.setUseParticleAnalyzer (false);
         f.UseRectAnalyzer=false;
         f.ScanSeparateRandomSubAreas=false;
         f.showcolours=false;
         f.RandomMassSample=false;                    
         f.setFill (false);
         f.DisableRoiManager=false;    
         f.GridPositions=1;
//get user inputs and set variables
//neutralize subscan settings then get the
//get type of sub area scan to do                
         f.SubScanType=sd.getNextChoiceIndex ();   
         f=ChooseSubScanTypeOptions(f);
         if (f.ScanSeparateRandomSubAreas||
                 f.UseRectAnalyzer||f.UseParticleAnalyzer )
         {
             f.MultiFiles=false;
         }
         else f.MultiFiles=true;
//get type of image to scan
         f.setImageType (sd.getNextChoiceIndex ());
         f.CheckPixRatio=sd.getNextBoolean ();
         f.BLOCKTEXTUREANALYSIS= sd.getNextBoolean ();
         f.showGrids=sd.getNextBoolean ();     
//get background colour                
         f.ChooseBackground= sd.getNextChoiceIndex ();
         if(f.ChooseBackground==f.USEBLACK)
         { 
             f.background=f.BLACK;
             f.foreground=f.WHITE;
         }
         if(f.ChooseBackground==f.USEWHITE)
         {
             f.background=f.WHITE;
             f.foreground=f.BLACK;
         }                
//get options for the box sizes
         f.Series=sd.getNextChoiceIndex ();
         f.setMaxNumBoxSizes ((int)sd.getNextNumber ());
         if(f.getMaxNumBoxSizes ()<=0)
         {
             f.setMaxNumBoxSizes (0);
             f.setUserWantsToSpecifyNumberOfBoxes(false);
         }
         else f.setUserWantsToSpecifyNumberOfBoxes (true);
         f.UseGreater=sd.getNextBoolean ();
         //subbox, maxpercent
         f.Subboxsize=(int)sd.getNextNumber ();
         f.maxpercent=(float)sd.getNextNumber();
         f.setUserMaxBoxSize ((int)sd.getNextNumber ());                 
         f.setUserMinBoxSize ((int)sd.getNextNumber ());
         if (f.getUserMinBoxSize ()<=1) f.setUserMinBoxSize (1); 
//get options for viewing the results
         f.ShowData = sd.getNextBoolean ();
         f.WriteRawData = sd.getNextBoolean ();
         f.printFrequencies = sd.getNextBoolean ();
         f.MaxFrequencies = (int)sd.getNextNumber ();
         if (f.MaxFrequencies<5)f.MaxFrequencies = 5;
//get graphing options
         f.setGraphRegression (sd.getNextBoolean ());
         f.setGraphLac(sd.getNextBoolean());
//get hull and circle options 
         f.setDocircul (sd.getNextBoolean ());
         f.DRAWHull=sd.getNextBoolean ();
         f.DRAWCircle=sd.getNextBoolean ();
         if (f.DRAWCircle||f.DRAWHull)f.setDocircul (true);
//set grid and gray
         f.Globalxy=new int[f.GridPositions][2];
         for (int i = 0; i < f.GridPositions;i++)
         {
             f.Globalxy[i][0]=0;
             f.Globalxy[i][1]=0;
         }
         if(f.getImageType ()==f.GRAY)
         {
             f.setDocircul (false);
             f.DRAWHull=false;
             f.DRAWCircle=false;
         }
         if(f.ScanSeparateRandomSubAreas)
         {
             f.subsamples= (int)IJ.getNumber(
                     "Number of random samples", f.subsamples);
             if(f.subsamples<1)f.subsamples=1;
             //set the pixel ratio for random scans
             f.setFACTOR (getFactor (f.FACTOR));
         }
     }
     f.proceed=true;
     if(true)
     {        //set graphics options           
       GenericDialog gd = new GenericDialog ("Sub Scan Options",
               IJ.getInstance ());
       String [] Composites = new String[13];
       for (int i = 0; i < Composites.length; i++)Composites[i]
               =Integer.toString (i);
       gd.addChoice ("Composite type (default 5)", Composites,
               Composites[f.getComptype ()]);
       String [] ColourSchemes = new String [21];
       for (int i = 0; i < ColourSchemes.length; i++)
           ColourSchemes[i]=Integer.toString (i);
       gd.addChoice ("Colour scheme (11 to customize):",
               ColourSchemes, ColourSchemes[f.getColourcode ()]);
       gd.addSlider ("Opacity", 0.0, 255.0, f.LocalAlpha);
//get the transparency and colour codes for the drawing
       gd.showDialog ();
       if (gd.wasCanceled ())
       {
           f.proceed=false;
           return f;
       }
       else{
           f.setComptype (gd.getNextChoiceIndex ());
           f.setColourcode (gd.getNextChoiceIndex ());
           if(f.getColourcode ()==11) getUserColourGuides (f);
           f.LocalAlpha=(int)gd.getNextNumber ();
       }  
       if (f.isGray ()) f.grayscantype=set2DGray (f.grayscantype); 
       if(f.GraphLac)f.SameScale=setSameScale();
       if(f.Series==f.POWERSERIES)
           f = getPowerSeriesOptions (f);
       else if (f.Series==f.SCALEDSERIES)
           f=GetScaledSeries (f);   
       else if (f.Series==f.RELATIVESERIES)
           f=  showRelativeMessage(f);
     }
     f.proceed=true;return f;
 }//end function
 
 
 
 public static FLVars ChooseSubScanTypeOptions(FLVars f)
 {
           switch(f.SubScanType)
                {
                    case 0:
                    {
                            f.setUseParticleAnalyzer (true);
                            f.RoiColouringMethod
                            =FLGraphics.
                            DRAW_IRREGULAR_ROI_NO_STRING;
                            f.showcolours=false;
                            break;
                    }
                    case 1:
                    {
                            f.setUseParticleAnalyzer (true);
                            f.RoiColouringMethod
                            =FLGraphics.FILL_IRREGULAR_ROI;
                            f.showcolours=true;
                            break;
                    }
                    case 2:
                    {
                            f.setUseParticleAnalyzer (true);
                            f.RoiColouringMethod=FLGraphics.
                            DRAW_IRREG_ROI_WRITE_STRNG;
                            f.showcolours=false;
                            break;
                    }
                    case 3:
                    {
                            f.UseRectAnalyzer=true;
                            f.RoiColouringMethod =FLGraphics.
                            DRAW_RECTANGLE_WRITE_STRING;
                            break;
                    }
                    case 4:
                    {
                            f.UseRectAnalyzer=true;                         
                            f.RoiColouringMethod
                                =FLGraphics.FILL_RECTANGLE;
                            break;
                    }
                    case 5:
                    {
                            f.UseRectAnalyzer=true;
                            f.RoiColouringMethod =FLGraphics.
                            REPLACE_FRGRND_PIX_RECT_ROI;
                            break;
                    }
                    case 6:
                    {
                            f.RoiColouringMethod
                            =FLGraphics.FILL_RECTANGLE;
                            f.showcolours=true;
                            f.ScanSeparateRandomSubAreas=true;
                            f.setFill (true);
                            f.DisableRoiManager=true;
                            break;
                    }
                    case 7:
                    {
                            f.showcolours=true;
                            f.RoiColouringMethod
                            =FLGraphics.
                            REPLACE_FRGRND_PIX_RECT_ROI;
                            f.ScanSeparateRandomSubAreas=true;
                            f.DisableRoiManager=true;
                            break;
                    }
                    case 8:
                    {
                            f.ScanSeparateRandomSubAreas=true;
                            f.DisableRoiManager=true;
                            f.RoiColouringMethod=FLGraphics.NO_COLOUR;
                            break;
                    }                         
                    default:
                    {
                            f.setUseParticleAnalyzer (true);
                            f.RoiColouringMethod
                            =FLGraphics.
                            DRAW_IRREGULAR_ROI_NO_STRING;
                            f.showcolours=false;
                            break;
                    }
                }
     
     return f;
 }

 
static float getFactor(float def)
{
    GenericDialog gd = new GenericDialog("Set Pixel Ratio");
    gd.addNumericField ("Pixel Ratio", def, 6, 1, "");
    gd.showDialog();
    if (gd.wasCanceled ()) return def;
    def = (float)gd.getNextNumber();
    if (def<0f)def=0f;
   return def;
}

/**Returns true if the user wants to use the 
* {@link FLVars#SameScale same scale} on all 
*{@link FLMain#graphL lacunarity plots}
* , or false if the user wants to use a different
*scale for each graph of an image
*@return true to use the same scale or else false*/
static boolean setSameScale()
{
return IJ.showMessageWithCancel ("Scale",
         "Use the same scale on all plots?"); 
}     
/**Returns an int for whatever method the user 
* chooses.  
*@see FLVars#DIFFERENTIALBC
*@see FLVars#VARIATION2DBC
*@param f = int for type of grayscale scan
* @return int for the type of {@link FLVars#grayscantype 
scan} for grayscale images*/
static int set2DGray(int f)
{
String current =(f==FLVars.DIFFERENTIALBC)?
"Differential Box Counting Method":"2d Variation";
String  other = (f==FLVars.DIFFERENTIALBC)?
"2d Variation":"Differential Box Counting Method";
boolean keep =IJ.showMessageWithCancel("Grayscale scan",
"Use "+current+"? (cancel to use "+other+".)");
int t=f;
if(!keep)
{  t=(f==FLVars.DIFFERENTIALBC)?
FLVars.VARIATION2DBC:FLVars.DIFFERENTIALBC;}
return t;    
}
/**
* Gets the user's choices for colours to 
* draw colour-coded sub scan images.
* Opens a {@link GenericDialog dialog} 
* that asks the user to set several sliders
* that are read to fill the
* {@link  FLVars#UserColours array of floats}representing
*  colour coding by fractal dimensions.
* If the dialog is cancelled, nothing is changed.
*
* @param f FLVars instance of variables
*/
public static void getUserColourGuides (FLVars f)
             {
                 GenericDialog sd
                         = new GenericDialog
                         ("Colour Levels",
                         IJ.getInstance ());
                 sd.addMessage (
                         "Fractal dimension limits for separating colours"+
                         " (Dimension * 100)");
                 for (int i = 0; i < f.UserColours.length; i++)
                     sd.addSlider ("Index "+ (i+1), 0, 300, 
                             100*f.UserColours[i]);
                 sd.showDialog ();
                 if (sd.wasCanceled ())
                 {
                 }
                 else
                 {
                     for (int i = 0; i < f.UserColours.length; i++)
                         f.UserColours[i]=((float)sd.getNextNumber ()/100);
                 }
             }
             
             
             
}


