// Notice
//  
// The contents of this file are subject to the
// same license as the source code for ImageJ.
// FracLac is in the public domain.  
// If you use it to publish research, please cite the source.
// The initial developer of the original 
// code is Audrey Karperien, Charles Sturt University.
package FLUtilities;
   
import ij.*;

  
   
/**Calculates basic statistics for arrays of floats or integers and 
 * probability density distributions for arrays. Uses 0 as lower limit; 
 * largest value in array as upper limit for binned distributions.
 * <h3>Inputs:</h3>
 * <ul><LI> arrays of floats or integers </LI>
 * <LI>integer for number of elements to
 * use from this array</LI></ul>
 * <h3>Outputs:</h3>
 * <ul><LI>floats that are accessible by instantiating
 * this class;
 * <LI>Variance, </LI><LI>mean</LI>, <LI>coefficient of
 * variation,</LI> <LI>and standard deviation</LI> of
 * an array as a sample as well as <LI>probability distribution
 * for a passed array of integers</LI>.</LI></ul>
 * <h3>Structure:</h3>   Default constructor does nothing.  Other
 * constructors call 
 * {@link #CalculateUsingInternalVariables calculations}
 * automatically
 * which sets variables ({@link #stddev}, {@link #mean}, 
 * {@link #coefVar},
 *{@link #variance}). 
 *
 *<ul><LI></p>
 *file:  statistics.java
 *
 * @version see FracLac_2.4e
 * @author Audrey Karperien Charles Sturt University
 */
public class FLstatistics 
{ 
    /**
     * 
     * @param args 
     */
    public static void main(String []args)
    { float sum = 0f;
      float mean = 0f;
      int bins = 4;
        float array []={0,25f, 50f, 75f, 100f,25f, 50f, 75f, 100f,
        };
        float [][] a = binnedProbabilityDistribution
                (array, array.length, bins, true);
        for (int i = 0; i < a[0].length; i++)
           {System.out.println(a[0][i]+ ", "+a[1][i]+"  "+ a[0][i]*a[1][i]);
        sum=sum+a[1][i];
        mean = mean +a[1][i]*a[0][i];}       
         
        System.out.println(sum+"  " + "mean "+mean);
        sum=0;
        mean=0;
        a = binnedProbabilityDistribution
                (array, array.length, bins, false);
        for (int i = 0; i < a[0].length; i++)
           {System.out.println(a[0][i]+ ", "+a[1][i]+"  "+ a[0][i]*a[1][i]);
        sum=sum+a[1][i];
        mean = mean +a[1][i]*a[0][i];}
       
         sum = sum;
        System.out.println(sum+"  " + "mean "+mean);
    }
        
    /**Standard deviation = &#8730;({@link #variance})*/
    public float stddev = 0f;
    /**Coefficient of variation={@link #stddev}/{@link #mean}*/
    public float coefVar = 0f;
    /**CV<sup>2</sup>(CV=Coefficient of variation=
     * standard deviation/mean = &sigma;/&mu;)
     @see #variance
     @see #mean
     @see #stddev*/
    public float cvsq = 0f;
    /**variance = (&#8721;X<sup>2</sup>
     * - (&#8721;X)<sup>2</sup>/N)/(N-1)*/
    /**variance = (&#8721;X<sup>2</sup>
     * - (&#8721;X)<sup>2</sup>/N)/(N)*/
    public float variance = 0f;
    /**mean = &#8721;x/N (N=number of x values)*/
    public float mean = 0f;
   public float sum = 0f;
    /**Constructor accepts no arguments; instantiates only.*/
    public FLstatistics()
    {};
/**Constructor takes array to test and number of values to test.
* Calls {@link #CalculateUsingInternalVariables calculate} method.
* @param XArray float[] array of floats to assess
* @param NUM integer for number of values to use from array*/
public FLstatistics(float[] XArray, int NUM) 
{       
        sum=0f;
        for (int i = 0; i < NUM; i++)
        {
            sum=sum+XArray[i];
        }
        CalculateUsingInternalVariables(XArray, sum, NUM);
    }
    
/**Constructor takes array to test and number of values to test.
* Calls {@link #CalculateUsingInternalVariables calculate} method.
* @param xint array of integers to assess
* @param NUM integer for number of values to use from array*/
public FLstatistics(int[] xint, int NUM)
{
        float [] Xfloat=new float[NUM];
        float sumx=0f;
        for (int i = 0; i < NUM; i++)
        {
            Xfloat[i]=(float)xint[i];
            sumx=sumx+Xfloat[i];
            
        }
        
        float num=(float)NUM;
        CalculateUsingInternalVariables(Xfloat, sumx, NUM);
    }
    
/**Calculates statistics on passed array using previously set number
 * (see Constructors)
* @param SUMX float for the sum of the array
* @param num integer for the number of elements to assess
* @param x array of floats to assess*/
public void CalculateUsingInternalVariables(float[] x,
        float SUMX, int num)
{
        float sumxsq=0f;
        float N = (float)num;
        
        for (int i =0;i<num; i++)
        {
          sumxsq=sumxsq+(x[i]*x[i]);            
        }
        variance =   ((sumxsq-((SUMX*SUMX)/N))/(N));
         if(!(variance>=0))variance=0f;
        
        stddev = (float)Math.sqrt(variance);
        mean = SUMX/N;
        cvsq=variance/(mean*mean);
        coefVar=stddev/mean;       
    }
    
    //////////////////////////////////////////////////////////////////////////
    /** Returns the standard deviation (&sigma;)
     * of the passed array, as
     * the square root  of the return from the 
     * {@link #Variance Variance} method
     * (&#8731;&sigma;<sup>2</sup>).
     * @param x [] floats to assess
     * @param n int for number of elements in x to assess
     * @return float for standard deviation of passed array
     */
    public static float StdDev(float[] x, int n)
    {
        float [] LV = Calc(x, n);
        return LV[1];   
         
    }
    
    //////////////////////////////////////////////////////////////////////////
    /** Returns the coefficient of variation, CV, of the passed array, as
     * the {@link #StdDev &sigma;}/{@link #Mean &mu;}.
     * @param x [] floats to assess
     * @param n int for number of elements in x to assess
     * @return float for coefficient of variation
     */ 
    public static float CoefVar(float[] x, int n)
    {
        float [] LV = Calc(x, n);
        return LV[3];                  
    }
    
    //////////////////////////////////////////////////////////////////////////
    /** Returns the mean of the passed array, as
     * (&#8721;x)/n
     * @param x [] floats to assess
     * @param n int for number of elements in x to assess
     * @return float for the mean
     */
    
    public static float Mean(float[] x, int n)
    {
        float [] LV = Calc(x, n);
        return LV[2];  
    }
    
    //////////////////////////////////////////////////////////////////////////
    /** Returns the variance of the passed array as
     * (&#8721;(x<sup>2</sup>)-((&#8721;x)<sup>2</sup>/N))/(N)
     * @param x float [] to assess
     * @param n int for number of values to assess
     * @return float for the variance of the array
     */
    public static float Variance(float[] x, int n)
    {
        float [] LV = Calc(x, n);
        return LV[0];          
    }
   
    /**Returns statistics for the first n elements of the
     * passed array, M.
     *
     *<li>&sigma;<sup>2</sup>=
     * Variance =((&#8721;M<sup>2</sup>
     * -((&#8721;M*&#8721;M)/N))/(N));
        vals[0]=Variance;        
        
        <li>&sigma; = Standard Deviation = stddev = 
     * &#8730; Variance;
        vals[1]=&sigma;
        
        float Mean = SumM/N;
        vals[2] = &mu; 
        
        float CoefVar=Stddev/Mean;
        vals[3]=CoefVar;       </pre>
     **/
    static float[] Calc (float[] M, int n)
    {
        float SumMsq=0;
        float SumM=0;
        float SumMM=0;
        float SumPsq=0;
        float N = (float)n;
        
        for (int i =0;i<n; i++)
        {
            SumM=SumM+M[i];
            SumMsq=SumMsq+(M[i]*M[i]);
            
        }
        for (int i = 0; i < n; i++)
        {
            SumMM = (M[i]*M[i])*(M[i]/SumM);
        }
        
        float [] vals = new float [4];
        
        float Variance =((SumMsq-((SumM*SumM)/N))/(N));
        //float Variance =((SumMsq-((SumM*SumM)/N))/(N-1));
        vals[0]=Variance;        
        
        float Stddev =(float) Math.sqrt(Variance);
        vals[1]=Stddev;
        
        float Mean = SumM/N;
        vals[2] = Mean; 
        
        float CoefVar=Stddev/Mean;
        vals[3]=CoefVar;        
        
        return vals;        
        
    }

    
    /**Returns the mean (&mu;), standard deviation (&sigma;), and 
     * Coefficient of Variation squared (&sigma;/&mu;)
     * <sup>2</sup>
     * for the passed array's first NumElements, using a
     * probability distribution from 0 to Max (i.e., if Max = 3, then
     * the histogram has bins at 0, 1, 2, and 3). If NumElements
     *is greater than the passed array's length, the array is 
     *padded with zeros to make up the difference.
     @param Masses float [] Masses array 
     @param NumElements float for the number of elements to use
     in the array
     @param Max int for the maximum value 
     @return {mean, stdev, cv}*/
    public static float []
          CalculateCVsqForMassesAtEpsilon(float []   Masses,
            float Max, float NumElements)
    {        
        
        float [] probability =
                CalculatePaddedProbabilityDistribution (Masses,  
                 Max, (int)NumElements);   
        
        float mu = 0;//initialize to zero as there are no negatives
        for (int i = 0; i <probability.length; i++)
            mu = mu + (i*probability[i]);        
     
        float sigmasquared = 0;
        for ( int i = 0 ; i < probability.length; i ++)
            sigmasquared
                = sigmasquared + (((i-mu)*(i-mu))*probability[i]);
        float sigma = (float)Math.sqrt(sigmasquared);        
        float cvsq = (float)(sigmasquared)/(mu*mu);        
        
        //return (cvsq);
        return new float []{mu, sigma, cvsq};
        
    }
    /**Returns a probability distribution for the passed  
     *array, where each index is the amount and the value
     * stored at that index is the probability of that amount.
     * First makes
     *a frequency histogram by counting how many of 
     * each value occurred in the array. 
     * The maximum value in the passed array determines 
     *the number of bins (elements) in the histogram, and the
     * increment between bins is one, represented by the
     *  index of the frequency (e.g., if the value at index 0 is 4, that
     * means there were four zeros in the passed array).    Then, 
     *the frequency is divided by the <i>TotalEntries</i>, 
     *the value passed along with the integer array.  TotalEntries
     *is not necessarily the same as the length of the array, but
     *  usually should be.    If it is greater than the length of the array,
     *then the arrays is padded with zeros up to NumElements.
     *@param mass float [] >0 to find the distribution for
     *@param NumElements int for the number of entries to consider;
     *once the frequency histogram is made, each value is
     *divided by TotalEntries to make the probability distribution;
     *it is usually the same as the length of the array, but may
     *be used to account for zeros that were excluded from 
     *the passed array (i.e., NumElements = mass[].length+zeros)
     *@return float [] for the probability distribution
     **/
 public static float [] CalculateProbabilityDistribution(float [] 
            mass, float NumElements)
    {
        int Max = (int) (FLutil.maxArray (mass));        
        float frequency [] = new float [Max+1];
        for (int i = 0; i < frequency.length; i++)frequency[i]=0f;
        for (int i = 0; i < mass.length; i++)
        {//count how many of each number there were
            frequency[(int)mass[i]]++;
        } 
        float TotalPlusZeros = NumElements>mass.length?
                    (float)NumElements: mass.length;
            if (NumElements>mass.length)
                frequency[0]=frequency[0]+(float)
                (NumElements-mass.length);
        
        float probability [] = new float [frequency.length];
        for (int i = 0; i <frequency.length;  i ++)
            probability[i]=frequency[i]/TotalPlusZeros; 
        return probability;
 }

 /**Returns the frequency histogram for the 
  * passed array of ints assuming the highest value of interest 
  * is max and the lowest is zero.    
  * The number of each value that occurred in all is stored in
  * a histogram at the index corresponding
  *to the value being counted.  That is, if there are three instances 
  *of 221 in the entire mass array, then at index 221 the value 
  *will be 3 in the frequency array.  If max is 5, then 
  *the histogram will have 6 bins (i.e., 0, 1, 2, 3, 4, and 5).  
  *@param mass int [] to arrange
  *@param max float for maximum value to include in the histogram  
  *@return float [] that holds the frequencies of each element up 
  *to the value of max*/
 public static float [] CalculateFrequencyDistribution(float [] 
            mass, float max)
    {               
        float frequency [] = new float [(int)max+1];
        for (int i = 0; i < frequency.length; i++)frequency[i]=0f;
        for (int i = 0; i < mass.length; i++)
        {//count how many of each number there were
            frequency[(int)mass[i]]++;} 
        return frequency;
 }
 /** Returns the padded probability distribution for the passed 
  array of integers&>;=0, counting only values &<;=max. 
  * NumElements is
  *for padding with zeros, but if NumElements=mass.length,
  * then nothing is changed.  If NumElements&>;mass.length, then
  *the frequency at index 0 is increased: 
  * <li>F[0]=F[0]+(NumElements-mass.length)</li>
  * @param NumElements float for the number of elements
  *where if mass.length is less than NumElements, the frequency 
  *histogram is increased at 0 by the difference.
  *@param maxMass int for the greatest value that should be
  * considered in the passed array; the histogram will have 
  *maxMass+1 bins (e.g., if maxMass is 3, the bins are 0, 1, 2, and 3)
  *@param mass int [] of values to distribute
  @return float [] that is the frequency distribution divided
  by the total number of elements in the original array,
  or else NumElements if NumElements>mass.length*/
 public static float [] CalculatePaddedProbabilityDistribution(float [] 
            mass, float maxMass, int NumElements)
    {               
        float [] frequency =
                CalculateFrequencyDistribution( mass, maxMass);     
        float TotalPlusZeros = NumElements>mass.length?
                    NumElements: mass.length;
            if (NumElements>mass.length)
                frequency[0]=frequency[0]+(NumElements-mass.length);
        float probability [] = new float [(int)maxMass+1];
        for (int i = 0; i <frequency.length;  i ++)
            probability[i]=frequency[i]/TotalPlusZeros; 
        return probability;
 }
 /**Returns the moments of the passed probability distribution,
  *in which the index corresponds to the value of each entry, and
  * each entry is the probability.  For instance, this is the array
  *made when FracLac scans a sliding box lacunarity scan,
  *so that the array entries are the number of
  *pixels that were in a box that was
  * slid over an image at several overlapping locations.
  * To make the probability distribution,
  * see {@link #CalculateProbabilityDistribution PD function}. The
  *mean &mu; is calculated as &#8721;[X*P[x]], and
  *&sigma;<sup>2</sup> is &#8721;[(X-&mu;)
  * <sup>2</sup>*P[x]]
  *then the cv is  &sigma;/&mu;. 
  * @param probability float [] of probabilities
  *@param moment
  * int for the value to return (0 for &mu; (mean), 1 for
  *&sigma; (standard deviation), or anything else 
  * for the cv (coefficient of
  *variation=&sigma;/&mu;).
  * @return float if m=0, mean of the passed probability distribution,
  *if m=1, standard deviation (&#8730; of variance),
  * or m=2, the coefficient of variation, or else the variance*/
 public static float MomentsOfProbabilityDistribution (
         float [] probability,
         int moment)
 {
     float mu = 0;//initialize to zero as there are no negatives
        for (int i = 0; i <probability.length; i++)
            mu = mu + (i*probability[i]);   
        float sigmasquared = 0;
        for ( int i = 0 ; i < probability.length; i ++)
            sigmasquared
                = sigmasquared + (((i-mu)*(i-mu))*probability[i]);
        float sigma = (float)Math.sqrt (sigmasquared); 
        float cv = sigma/mu;    
        return 
                moment==0?mu:
                    moment==1?sigma:
                        moment==2?cv:
                            sigmasquared;
    }


    
    

//    /**
//     * Returns the mean of the probability distribution.
//     * @param F float [] to analyze
//     * @return float for mean
//     */
//    public static float MeanOfProbabilityDistribution (float [] F)
//    {
//        float [] P = ProbabilityArray(F);  
//        float SumOfXTimesProbability = 0;
//        for (int i = 0; i < F.length; i++)
//            SumOfXTimesProbability=
//                    SumOfXTimesProbability+F[i]*P[i];
//        return SumOfXTimesProbability;
//    }
//      
   
    /**
   * Returns an array the sum of which is one, 
     * holding a probability for each integer  
     *from 0 to some value equal to the 
     * length of the array.   The frequency or number of
   * instances of each value is passed in the original 
     * array entry, and the value itself 
   * is the index in the passed array.  That is, the 
     *returned array has elements P[i] =F[i]/&#8721;F.
   * @param F float [] of values
   * @return float [] of probabilities
   */
  public static float [] ProbabilityArray(float [] F)
    {
        float [] P = new float [F.length];
        float sum=0;
        for (int i =0; i < F.length; i++)sum=sum+F[i];
        for (int i =0; i < F.length; i++)
        P[i]=F[i]/sum;
        return P;
        
    }
    /**Returns the sum of the elements in the passed array, up
     *to the number passed.
     *
     *@param array int [] array to sum
     *@param Nums int for number of elements to sum
     *@return float for sum of all elements*/
    public static float SumArray(int [] array, int Nums)
    {
        float sum=0;
        for (int i =0; i < Nums; i++)sum=sum+array[i];
        return sum;
    }
    
     /**returns the sum of the elements in the passed array, up
     *to the number passed.
     *
     *@param array int [] array to sum
     *@param Nums int for number of elements to sum
     *@return float for sum of all elements*/
    public static float SumArray(float [] array, int Nums)
    {
        float sum=0;
        for (int i =0; i < Nums; i++)sum=sum+array[i];
        return sum;
    }

    /**Returns an array holding the number of values that fell at each
     *position in the passed bins array.  So, if you pass a bins array
     * having {2.0, 4.0, 6.0} and a large array of values, then
     * are returned an array of {12, 45, 16} that means there
     * were 12 at weight 2, 45 at weight 4, and 16 at weight 6.
     * @return integer array of frequency (count) of newarray
     * members for each value in bins array
     * @param array float array of values to count and sort
     * @param bins float array of values to use for the bins
     * from the passed array*/
    public static int[][] CountExactElements(int [] array,
    int [] bins)
    {
        int []counts = new int [bins.length];
        for (int i = 0; i<bins.length; i++)
            counts[i]=0;
        
        for (int b = 0; b<array.length;b++)
        {
            
            for (int a = 0; a <= bins.length; a++)
            {
                
                if (a==bins.length)
                {
                    {int [] temp = new int[bins.length];
                     int [] tempc = new int [bins.length];
                     for (int i = 0; i < temp.length; i++)
                     {  temp[i]=bins[i];
                        tempc[i]=counts[i];
                     }
                     bins=new int[bins.length+1];
                     counts = new int [bins.length+1];
                     for (int d = 0; d < temp.length; d++)bins[d]=temp[d];
                     
                     for (int i = 0; i < temp.length; i++)
                     {
                         bins[i]=temp[i];
                         counts[i]=tempc[i];
                     }
                     bins[bins.length-1]=array[b];
                     counts[counts.length-1]=1;
                    }}
                
                
                else if ((a<bins.length)&&(array[b]==bins[a]))
                {
                    counts[a]++;
                    a=bins.length+1;
                }
                
            }
        }
        
        return new int[][]
        {counts, bins};
        
    }
    
    
    /**
     * Returns the probability 
     * distribution of a passed array of floats greater than 
     * or equal to 0, using the
     * passed number of bins. The sum of the
     * returned array [0] is 1.0.  The returned array[1] is the
     * bins, so that the mean of the distribution is 
     * the sum of all array[0][i]*array[1][i]. 
     * Counts the number of times each number
     * occurs in the entire array and stores the count for each
     * number at the corresponding position in a new array.
     * Divides each element by the passed number of 
     * counts to make an array
     * of the probability of each element in the original array.  If the 
     * NumElements <array.length, it is ignored, but if NumElements
     * >array.length, zeros are added to the histogram to make 
     * up the difference.
     * @return float [][] array probability distribution of passed array
     * array[0]=BINS, array[1]=Probabilities
     * @param bins int for the
     * number of bins to divide the distribution into
     * @param array [] array of floats >=0 to be distributed
     * @param NumElements number of elements to use; 
     * this value is ignored if less than the passed array's
     * length and if greater is used to pad the histogram with zeros
     */
    public static float[][] binnedGrayProbabilityDistribution(
    float [] array, int NumElements, int bins)
    {   int count =0;        

            float BiggestNumber=FLutil.maxArray (array);
            float SmallestNumber=FLutil.minArray (array);

            float binsize =BiggestNumber/(bins);
            
            float halfa = (float)binsize/2f;
            float BINS[]= new float[bins];
            BINS[0]=(halfa>0)?halfa: 1;
            for (int j = 1; j<(BINS.length); j++)
                BINS[j]=BINS[j-1]+binsize;
            
            float [] counts = FLutil.newArray (BINS.length, 0f);
            for (int k = 0; k <array.length; k++)
            {
                    for (int i = BINS.length-1; i>=0; i--)
                    {
                        //check each member of the array
                        //if it is less than or equal to the bin, put it in
                        if (array[k]>=(BINS[i]-halfa) )
                        { 
                            counts[i]++;
                            i=-1;
                        }
                    }
            }
            
            //now there is a frequency histogram that 
            //corresponds to BINS and it has to be made into
            //a probability distribution by dividing by the number of 
            //elements
            float TotalPlusZeros = NumElements>array.length?
                    NumElements: array.length;
            if (NumElements>array.length)
                counts[0]=counts[0]+(NumElements-array.length);
            //if there are zeros to be added, add them now        
            float P [] = new float [BINS.length];
            for (int i = 0; i < P.length; i++)
                P[i]=counts[i]/TotalPlusZeros;
            
           
        return new float [][]{BINS, P};
        
    }
    static boolean debug=false;
    static void debug(int s)
    {
        if (debug)IJ.log("Statistics" + Integer.toString(s));
    }
    
    
        /**Finds the probability distribution for the first 
         * <i>NumElements</i> of the
         * passed array, using only those values 
         * that are greater than 0 if gray is false. 
         * The returned array[0][] is the
         *  values for the bin midpoints, and the returned
         *  array[1][] is the probabilities, where the sum of 
         * array [1][0 to end] is 1.
         * See code for calculations.
         * 
         * @param array [] floats to be binned
         * @param NumElements int for number of float elements
         *to bin and distribute
         * @param bins int for the number of bins to use in the 
         *distribution
         * @param gray boolean that switches to
         *{@link #binnedGrayProbabilityDistribution alternative}
         *method if true 
         * @return float [][] of probabilty distribution where
         *bins are at array[0][]  and probabilities are at array[1][]
         */
        public static float[][] binnedProbabilityDistribution(
    float [] array, int NumElements, int bins, boolean gray)
    {  
            
            if (gray) return  binnedGrayProbabilityDistribution (array, 
                        NumElements, bins);
            
            else
            {int count =0;
        
        for (int i = 0; i <NumElements; i++)
        {
            if (array[i]>0)count++;
        }
        
        float [] temparray = new float [count];
        int w = 0;
        for (int i = 0; i < NumElements; i++)
        {
            if (array[i]>0)
            {
                temparray[w]=array[i];
                w++;
            }
        }
        //NumElements = count;
        // if (count<bins)bins=count/15;
        float [][] newarray = null;
        // if (bins<15)bins=15;
        
        double fdd=0;
        if (temparray==null)return(null);
        if (temparray.length==0)return(null);
        if (count==0)return(null);
        else
        {
           
            float BiggestNumber=array[0];
            
            for (int i = 0; i<count; i++)
            {
                BiggestNumber=Math.max(BiggestNumber, temparray[i]);
            }
            if ((int)BiggestNumber<bins)bins=(int)BiggestNumber;
            int binsize = (int)BiggestNumber/bins;
            if(binsize<1)binsize=1;
            float halfa = (float)binsize/2;
            float BINS[]= new float[bins];
            BINS[0]=(halfa>0)?halfa: 1;
            for (int j = 1; j<(BINS.length); j++)
                BINS[j]=BINS[j-1]+binsize;
            int [] counts = new int [BINS.length];
            debug(344);//tally
            for (int i = 0; i < count; i++)
            {
                for (int b = 0; b < BINS.length;b++)
                {
                    {if (b>BINS.length-2)
                     {counts[b]++;b=BINS.length;}
                     else if (temparray[i]<BINS[b]+halfa)
                     {counts[b]++;b=BINS.length;}
                    }}}
            
            float []Prob=new float[counts.length];
            for (int i = 0; i < counts.length; i++)
            {
                Prob[i]=(float)counts[i]/(float)(count);
            }
            
            newarray = new float [2][BINS.length];
            for (int i = 0; i < BINS.length; i++)
            {
                newarray[0][i]=BINS[i];
                //System.out.println(BINS[i]);
                
                newarray[1][i]=Prob[i];
                fdd=fdd+Prob[i];
                //System.out.println(Prob[i]);
            }
        }//end if the array was not empty
        //System.out.println(fdd);
             return newarray;}
        
    }
    

    
    
    
}//end class statistics

