import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.plugin.filter.*;
import ij.plugin.*;
import ij.process.*;
import ij.gui.*;
import ij.measure.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.font.*;

/** This plugin calculates a two dimensional (2-D) gaussian lowpass filter using
     a 2-D Gaussian or Normal density function is defined in the literature.

     Gaussian filters are important in many signal processing, image processing,
     and communication applications. These filters are characterized by narrow bandwidths and
     sharp cutoffs. A key feature of Gaussian filters is that the Fourier transform of a Gaussian is
     also a Gaussian, so the filter has the same response shape in both the time and frequency domains.

     The cutoff parameter defines the cutoff-frequency. The DC-level parameter defines the height of the dc center component.

     Erik Lieng 01/10/2002
*/

public class Gaussian_Filter implements PlugIn {
    static double cutoff = 50;
    static double dc_level = 255;
    static final int width =256;
    static final int height = 256;

    public void run(String arg) {

        if (!showDialog())
            return;


        ByteProcessor img = new ByteProcessor(width,height);

        double value = 0;
        double distance = 0;
        int sizex = img.getWidth();
        int sizey = img.getHeight();
        int xcenter = (sizex/2)+1;
        int ycenter = (sizey/2)+1;
        for (int y = 0; y < sizey; y++){
            for (int x = 0; x < sizex; x++){
                distance = Math.abs(x-xcenter)*Math.abs(x-xcenter)+Math.abs(y-ycenter)*Math.abs(y-ycenter);
                distance = Math.sqrt(distance);
                value = dc_level*Math.exp((-1*distance*distance)/(1.442695*cutoff*cutoff));
                img.putPixelValue(x,y,value);

        }
        }

         ImagePlus img_filter2 = new ImagePlus("2D Filter",img);
         img_filter2.show();
   }

    public boolean showDialog() {
        GenericDialog gd = new GenericDialog("Parameters");
        gd.addNumericField("Cutoff frequency:", cutoff, 1);
        gd.addNumericField("DC-level:", dc_level, 1);
        gd.showDialog();
        if (gd.wasCanceled())
            return false;
        cutoff = gd.getNextNumber();
        dc_level = gd.getNextNumber();

        return true;
    }


}
